@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -32,9 +32,11 @@ from beartype.roar import (
     BeartypeCaveNoneTypeOrKeyException,
     BeartypeCaveNoneTypeOrMutabilityException,
 )
-from beartype._util.hint.nonpep.utilhintnonpeptest import die_unless_hint_nonpep
+from beartype._util.hint.nonpep.utilhintnonpeptest import (
+    die_unless_hint_nonpep)
+from typing import Union, Optional
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ CONSTANTS                         }....................
@@ -90,10 +92,10 @@ class _NoneTypeOrType(dict):
         '''
 
         raise BeartypeCaveNoneTypeOrMutabilityException(
-            '{!r} externally immutable (i.e., not settable).'.format(self))
+            f'{repr(self)} externally immutable (i.e., not settable).')
 
 
-    def __missing__(self, hint: (type, tuple)) -> tuple:
+    def __missing__(self, hint: Union[type, str, tuple]) -> tuple:
         '''
         Dunder method explicitly called by the superclass
         :meth:`dict.__getitem__` method implicitly called on getting the passed
@@ -150,12 +152,13 @@ class _NoneTypeOrType(dict):
         )
 
         # Tuple of types to be cached and returned by this call.
-        hint_or_none = None
+        hint_or_none: Optional[tuple] = None
 
         # If this key is a type...
         if isinstance(hint, type):
             # If this type is "NoneType", reuse the existing "_NoneTypes" tuple
             # containing only this type.
+            # I don't think mypy recognises _NoneType as None, and so gets confused
             if hint is _NoneType:
                 hint_or_none = _NoneTypes
             # Else, this type is *NOT* "NoneType". In this case, instantiate a
@@ -177,7 +180,7 @@ class _NoneTypeOrType(dict):
         # Nonetheless, raise a human-readable exception for sanity.
         else:
             raise BeartypeCaveNoneTypeOrKeyException(
-                '"NoneTypeOr" key {!r} unsupported.'.format(hint))
+                f'"NoneTypeOr" key {repr(hint)} unsupported.')
 
         # Return this new tuple.
         #
