@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -13,6 +13,7 @@ This private submodule is *not* intended for importation by downstream callers.
 '''
 
 # ....................{ IMPORTS                           }....................
+from beartype.cave import NoneType, NoneTypeOr
 from beartype.roar import _BeartypeCallHintPepRaiseException
 from beartype._util.hint.pep.proposal.utilhintpep484 import (
     get_hint_pep484_newtype_class,
@@ -41,9 +42,9 @@ from beartype._util.hint.utilhinttest import (
     is_hint_forwardref,
     is_hint_ignorable,
 )
-from typing import NoReturn
+from typing import Any, Callable, NoReturn, Optional, Tuple
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ CLASSES                           }....................
@@ -65,14 +66,12 @@ class CauseSleuth(object):
         Human-readable label describing the parameter or return value from
         which this object originates, typically embedded in exceptions raised
         from this getter in the event of unexpected runtime failure.
-    func : object
+    func : Callable
         Decorated callable generating this type-checking error.
-    hint : object
-        Type hint to validate this object against.
-    hint_sign : object
+    hint_sign : Any
         Unsubscripted :mod:`typing` attribute identifying this hint if this hint
         is PEP-compliant *or* ``None`` otherwise.
-    hint_childs : Optional[tuple]
+    hint_childs : Optional[Tuple]
         Either:
 
         * If this hint is PEP-compliant:
@@ -85,8 +84,22 @@ class CauseSleuth(object):
             hint if this
 
         * Else, ``None``.
-    pith : object
+    pith : Any
         Arbitrary object to be validated.
+    random_int: Optional[int]
+        **Pseudo-random integer** (i.e., unsigned 32-bit integer
+        pseudo-randomly generated by the parent :func:`beartype.beartype`
+        wrapper function in type-checking randomly indexed container items by
+        the current call to that function) if that function generated such an
+        integer *or* ``None`` otherwise (i.e., if that function generated *no*
+        such integer). See the same parameter accepted by the higher-level
+        :func:`beartype._decor._code._pep._error.peperror.raise_pep_call_exception`
+        function for further details.
+
+    Attributes (Private)
+    ----------
+    _hint : Any
+        Type hint to validate this object against.
     '''
 
     # ..................{ CLASS VARIABLES                   }..................
@@ -97,10 +110,11 @@ class CauseSleuth(object):
         'cause_indent',
         'exception_label',
         'func',
-        'hint',
         'hint_sign',
         'hint_childs',
         'pith',
+        'random_int',
+        '_hint',
     )
 
 
@@ -110,27 +124,26 @@ class CauseSleuth(object):
         'func',
         'hint',
         'pith',
+        'random_int',
     ))
     '''
     Frozen set of the names of all parameters accepted by the :meth:`init`
     method, defined as a set to enable efficient membership testing.
     '''
 
-
-    _VAR_NAMES = frozenset(__slots__)
-    '''
-    Frozen set of the names of all instance variables permitted on this object,
-    defined as a set to enable efficient membership testing.
-    '''
-
     # ..................{ INITIALIZERS                      }..................
+    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+    # CAUTION: Whenever adding, deleting, or renaming any parameter accepted by
+    # this method, make similar changes to the "_INIT_PARAM_NAMES" set above.
+    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     def __init__(
         self,
-        func: object,
-        pith: object,
-        hint: object,
+        func: Callable,
+        pith: Any,
+        hint: Any,
         cause_indent: str,
         exception_label: str,
+        random_int: int,
     ) -> None:
         '''
         Initialize this object.
@@ -140,66 +153,104 @@ class CauseSleuth(object):
             f'{repr(cause_indent)} not string.')
         assert isinstance(exception_label, str), (
             f'{repr(exception_label)} not string.')
+        assert isinstance(random_int, NoneTypeOr[int]), (
+            f'{repr(random_int)} not integer or "None".')
 
         # Classify all passed parameters.
         self.func = func
         self.pith = pith
-        self.hint = hint
         self.cause_indent = cause_indent
         self.exception_label = exception_label
+        self.random_int = random_int
 
         # Nullify all remaining parameters for safety.
-        self.hint_sign = None
-        self.hint_childs = None
+        self.hint_sign: Any = None
+        self.hint_childs: Tuple = None  # type: ignore[assignment]
+
+        # Classify this hint *AFTER* initializing all parameters above.
+        self.hint = hint
+
+    # ..................{ PROPERTIES                        }..................
+    @property
+    def hint(self) -> Any:
+        '''
+        Type hint to validate this object against.
+        '''
+
+        return self._hint
+
+
+    @hint.setter
+    def hint(self, hint: Any) -> None:
+        '''
+        Set the type hint to validate this object against.
+        '''
 
         # ................{ REDUCTION                         }................
         #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         # CAVEATS: Synchronize changes here with the corresponding block of the
         # beartype._decor._code._pep._pephint.pep_code_check_hint() function.
         #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+        #
         # This logic reduces the currently visited hint to an arbitrary object
         # associated with this hint when this hint conditionally satisfies any
         # of various conditions.
         #
         # ................{ REDUCTION ~ pep 484               }................
+        # If this is the PEP 484-compliant "None" singleton, reduce this hint
+        # to the type of that singleton. While not explicitly defined by the
+        # "typing" module, PEP 484 explicitly supports this singleton:
+        #     When used in a type hint, the expression None is considered
+        #     equivalent to type(None).
+        if hint is None:
+            hint = NoneType
         # If this is a PEP 484-compliant new type hint, reduce this hint to the
         # user-defined class aliased by this hint. Although this logic could
         # also be performed below, doing so here simplifies matters.
-        if is_hint_pep484_newtype(self.hint):
-            self.hint = get_hint_pep484_newtype_class(self.hint)
+        elif is_hint_pep484_newtype(hint):
+            hint = get_hint_pep484_newtype_class(hint)
         # ................{ REDUCTION ~ pep 544               }................
         # If this is a PEP 484-compliant IO generic base class *AND* the active
         # Python interpreter targets at least Python >= 3.8 and thus supports
         # PEP 544-compliant protocols, reduce this functionally useless hint to
         # the corresponding functionally useful beartype-specific PEP
         # 544-compliant protocol implementing this hint.
-        elif is_hint_pep544_io_generic(self.hint):
-            self.hint = get_hint_pep544_io_protocol_from_generic(self.hint)
+        #
+        # Note that PEP 484-compliant IO generic base classes are technically
+        # usable under Python < 3.8 (e.g., by explicitly subclassing those
+        # classes from third-party classes). Ergo, we can neither safely emit
+        # warnings nor raise exceptions on visiting these classes under *ANY*
+        # Python version.
+        elif is_hint_pep544_io_generic(hint):
+            hint = get_hint_pep544_io_protocol_from_generic(hint)
         # ................{ REDUCTION ~ pep 593               }................
         # If this is a PEP 593-compliant type metahint, ignore all annotations
         # on this hint (i.e., "hint_curr.__metadata__" tuple) by reducing this
         # hint to its origin (e.g., "str" in "Annotated[str, 50, False]").
-        elif is_hint_pep593(self.hint):
-            self.hint = get_hint_pep593_hint(self.hint)
+        elif is_hint_pep593(hint):
+            hint = get_hint_pep593_hint(hint)
         # ................{ REDUCTION ~ end                   }................
 
         # If this hint is PEP-compliant...
-        if is_hint_pep(self.hint):
+        if is_hint_pep(hint):
             # Arbitrary object uniquely identifying this hint.
-            self.hint_sign = get_hint_pep_sign(self.hint)
+            self.hint_sign = get_hint_pep_sign(hint)
 
             # Tuple of either...
             self.hint_childs = (
                 # If this hint is a generic, the one or more unerased
                 # pseudo-superclasses originally subclassed by this hint.
-                get_hint_pep_generic_bases_unerased(self.hint)
-                if is_hint_pep_generic(self.hint) else
+                get_hint_pep_generic_bases_unerased(hint)
+                if is_hint_pep_generic(hint) else
                 # Else, the zero or more arguments subscripting this hint.
-                get_hint_pep_args(self.hint)
+                get_hint_pep_args(hint)
             )
 
+        # Classify this hint *AFTER* all other assignments above.
+        self._hint = hint
+
     # ..................{ GETTERS                           }..................
-    def get_cause_or_none(self) -> 'Optional[str]':
+    def get_cause_or_none(self) -> Optional[str]:
         '''
         Human-readable string describing the failure of this pith to satisfy
         this PEP-compliant type hint if this pith fails to satisfy this pith
@@ -392,18 +443,18 @@ class CauseSleuth(object):
 
         # For the name of each passed keyword argument...
         for param_name in kwargs.keys():
-            # If this copy does *NOT* already define an instance variable of
-            # the same name, raise an exception.
-            if param_name not in self._VAR_NAMES:
+            # If this name is *NOT* that of a parameter accepted by the
+            # __init__() method, raise an exception.
+            if param_name not in self._INIT_PARAM_NAMES:
                 raise _BeartypeCallHintPepRaiseException(
-                    f'Unrecognized instance variable '
-                    f'{self.__class__.__name__}.{param_name} not permutable.'
+                    f'{self.__class__}.__init__() parameter '
+                    f'{param_name} unrecognized.'
                 )
 
-        # For the name of each instance variable initializable by this class...
+        # For the name of each parameter accepted by the __init__() method...
         for param_name in self._INIT_PARAM_NAMES:
-            # If this variable is not already defined by these arguments,
-            # cascade the current value of this variable into these arguments.
+            # If this parameter was *NOT* explicitly passed by the caller,
+            # default this parameter to its current value from this object.
             if param_name not in kwargs:
                 kwargs[param_name] = getattr(self, param_name)
 
