@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -18,18 +18,19 @@ from beartype._decor._code._pep._error._peperrorsleuth import CauseSleuth
 from beartype._util.hint.data.pep.proposal.utilhintdatapep484 import (
     HINT_PEP484_SIGNS_UNION)
 from beartype._util.hint.pep.utilhintpepget import (
-    get_hint_pep_type_origin_or_none)
+    get_hint_pep_stdlib_type_or_none)
 from beartype._util.hint.pep.utilhintpeptest import is_hint_pep
 from beartype._util.text.utiltextjoin import join_delimited_disjunction_classes
 from beartype._util.text.utiltextmunge import (
     suffix_unless_suffixed, uppercase_char_first)
 from beartype._util.text.utiltextrepr import get_object_representation
+from typing import Optional
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ GETTERS                           }....................
-def get_cause_or_none_union(sleuth: CauseSleuth) -> 'Optional[str]':
+def get_cause_or_none_union(sleuth: CauseSleuth) -> Optional[str]:
     '''
     Human-readable string describing the failure of the passed arbitrary object
     to satisfy the passed PEP-compliant union type hint if this object actually
@@ -71,7 +72,7 @@ def get_cause_or_none_union(sleuth: CauseSleuth) -> 'Optional[str]':
         if is_hint_pep(hint_child):
             # Non-"typing" class originating this child hint if any *OR* "None"
             # otherwise.
-            hint_child_type_origin = get_hint_pep_type_origin_or_none(
+            hint_child_type_origin = get_hint_pep_stdlib_type_or_none(
                 hint_child)
 
             # If...
