@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -11,7 +11,60 @@ generated wrapper functions wrapping decorated callables).
 This private submodule is *not* intended for importation by downstream callers.
 '''
 
+#FIXME: Generalizing the "random_int" concept (i.e., the optional "random_int"
+#parameter accepted by the raise_pep_call_exception() function) that enables
+#O(1) rather than O(n) exception handling to containers that do *NOT* provide
+#efficient random access like mappings and sets will be highly non-trivial.
+#While there exist a number of alternative means of implementing that
+#generalization, the most reasonable *BY FAR* is probably to:
+#
+#* Embed additional assignment expressions in the type-checking tests generated
+#  by the pep_code_check_hint() function that uniquely store the value of each
+#  item, key, or value returned by each access of a non-indexable container
+#  iterator into a new unique local variable. Note this unavoidably requires:
+#  * Adding a new index to the "hint_curr_meta" tuples internally created by
+#    that function -- named, say, "_HINT_META_INDEX_ITERATOR_NAME". The value
+#    of the tuple item at this index should either be:
+#    * If the currently iterated type hint is a non-indexable container, the
+#      name of the new unique local variable assigned to by this assignment
+#      expression whose value is obtained from the iterator cached for that
+#      container.
+#    * Else, "None".
+#    Actually... hmm. Perhaps we only need a new local variable
+#    "iterator_nonsequence_names" whose value is a cached "FixedList" of
+#    sufficiently large size (so, "SIZE_BIG"?). We could then simply
+#    iteratively insert the names of the wrapper-specific new unique local
+#    variables into this list.
+#    Actually... *WAIT.* Is all we need a single counter initialized to, say:
+#        iterators_nonsequence_len = 0
+#    We then both use that counter to:
+#    * Uniquify the names of these wrapper-specific new unique local variables
+#      during iteration over type hints.
+#    * Trivially generate a code snippet passing a list of these names to the
+#      "iterators_nonsequence" parameter of raise_pep_call_exception() function
+#      after iteration over type hints.
+#    Right. That looks like The Way, doesn't it? This would seem to be quite a
+#    bit easier than we'd initially thought, which is always nice. Oi!
+#  * Python >= 3.8, but that's largely fine. Python 3.6 and 3.7 are
+#    increasingly obsolete in 2021.
+#* Add a new optional "iterators_nonsequence" parameter to the
+#  raise_pep_call_exception() function, accepting either:
+#  * If the current parameter or return of the parent wrapper function was
+#    annotated with one or more non-indexable container type hints, a *LIST* of
+#    the *VALUES* of all unique local variables assigned to by assignment
+#    expressions in that parent wrapper function. These values were obtained
+#    from the iterators cached for those containers. To enable these exception
+#    handlers to efficiently treat this list like a FIFO stack (e.g., with the
+#    list.pop() method), this list should be sorted in the reverse order that
+#    these assignment expressions are defined in.
+#* Refactor exception handlers to then preferentially retrieve non-indexable
+#  container items in O(1) time from this stack rather than simply iterating
+#  over all container items in O(n) brute-force time. Obviously, extreme care
+#  must be taken here to ensure that this exception handling algorithm visits
+#  containers in the exact same order as visited by our testing algorithm.
+
 # ....................{ IMPORTS                           }....................
+from beartype.cave import NoneTypeOr
 from beartype.meta import URL_ISSUES
 from beartype.roar import (
     BeartypeCallHintPepParamException,
@@ -36,7 +89,7 @@ from beartype._decor._code._pep._error._peperrorunion import (
 from beartype._util.hint.data.pep.utilhintdatapep import (
     HINT_PEP_SIGNS_SEQUENCE_STANDARD,
     HINT_PEP_SIGNS_TUPLE,
-    HINT_PEP_SIGNS_TYPE_ORIGIN,
+    HINT_PEP_SIGNS_TYPE_ORIGIN_STDLIB,
 )
 from beartype._util.hint.data.pep.proposal.utilhintdatapep484 import (
     HINT_PEP484_BASE_FORWARDREF,
@@ -49,9 +102,10 @@ from beartype._util.text.utiltextlabel import (
 )
 from beartype._util.text.utiltextmunge import suffix_unless_suffixed
 from beartype._util.text.utiltextrepr import get_object_representation
-from typing import Generic
+from collections.abc import Callable
+from typing import Generic, Optional
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ MAPPINGS                          }....................
@@ -79,9 +133,13 @@ excessively long as to prevent human-readability.
 
 # ....................{ RAISERS                           }....................
 def raise_pep_call_exception(
-    func: 'CallableTypes',
+    # Mandatory parameters.
+    func: Callable,
     pith_name: str,
     pith_value: object,
+
+    # Optional parameters.
+    random_int: Optional[int] = None,
 ) -> None:
     '''
     Raise a human-readable exception detailing the failure of the parameter
@@ -129,6 +187,27 @@ def raise_pep_call_exception(
           value returned from this callable.
     pith_value : object
         Passed parameter or returned value failing to satisfy this hint.
+    random_int: Optional[int]
+        **Pseudo-random integer** (i.e., unsigned 32-bit integer
+        pseudo-randomly generated by the parent :func:`beartype.beartype`
+        wrapper function in type-checking randomly indexed container items by
+        the current call to that function) if that function generated such an
+        integer *or* ``None`` otherwise (i.e., if that function generated *no*
+        such integer). Note that this parameter critically governs whether this
+        exception handler runs in constant or linear time. Specifically, if
+        this parameter is:
+
+        * An integer, this handler runs in **constant time.** Since there
+          exists a one-to-one relation between this integer and the random
+          container item(s) type-checked by the parent
+          :func:`beartype.beartype` wrapper function, receiving this integer
+          enables this handler to efficiently re-type-check the same random
+          container item(s) type-checked by the parent in constant time rather
+          type-checking all container items in linear time.
+        * ``None``, this handler runs in **linear time.**
+
+        Defaults to ``None``, implying this exception handler runs in linear
+        time by default.
 
     Raises
     ----------
@@ -152,6 +231,8 @@ def raise_pep_call_exception(
     '''
     assert callable(func), f'{repr(func)} uncallable.'
     assert isinstance(pith_name, str), f'{repr(pith_name)} not string.'
+    assert isinstance(random_int, NoneTypeOr[int]), (
+        f'{repr(random_int)} not integer or "None".')
     # print('''raise_pep_call_exception(
     #     func={!r},
     #     pith_name={!r},
@@ -159,10 +240,10 @@ def raise_pep_call_exception(
     # )'''.format(func, pith_name, pith_value))
 
     # Type of exception to be raised.
-    exception_cls = None
+    exception_cls: type = None  # type: ignore[assignment]
 
     # Human-readable label describing this parameter or return value.
-    pith_label = None
+    pith_label: str = None  # type: ignore[assignment]
 
     # If the name of this parameter is the magic string implying the passed
     # object to be a return value, set the above local variables appropriately.
@@ -180,21 +261,19 @@ def raise_pep_call_exception(
             param_value=pith_value,
         )
 
-    # PEP-compliant type hint annotating this parameter or return value if any
-    # *OR* "None" otherwise (i.e., if this parameter or return value is
-    # unannotated).
-    hint = func.__annotations__.get(pith_name, None)
-
     # If this parameter or return value is unannotated, raise an exception.
     #
     # Note that this should *NEVER* occur, as the caller guarantees this
     # parameter or return value to be annotated. Nonetheless, since callers
     # could deface the "__annotations__" dunder dictionary without our
     # knowledge or permission, precautions are warranted.
-    if hint is None:
+    if pith_name not in func.__annotations__:
         raise _BeartypeCallHintPepRaiseException(f'{pith_label} unannotated.')
     # Else, this parameter or return value is annotated.
 
+    # PEP-compliant type hint annotating this parameter or return value.
+    hint = func.__annotations__[pith_name]
+
     # If type hint is *NOT* a supported type hint, raise an exception.
     die_unless_hint(hint=hint, hint_label=f'{pith_label} type hint')
     # Else, this type hint is supported.
@@ -208,6 +287,7 @@ def raise_pep_call_exception(
         hint=hint,
         cause_indent='',
         exception_label=pith_label,
+        random_int=random_int,
     ).get_cause_or_none()
 
     # If this pith does *NOT* satisfy this hint...
@@ -217,7 +297,7 @@ def raise_pep_call_exception(
             text=exception_cause, suffix='.')
 
         # Raise an exception of the desired class embedding this cause.
-        raise exception_cls(
+        raise exception_cls(  # type: ignore[misc]
             f'{pith_label} violates type hint '
             f'{repr(hint)}, as {exception_cause_suffixed}'
         )
@@ -249,7 +329,7 @@ def _init() -> None:
     # Map each originative "typing" attribute to the appropriate getter
     # *BEFORE* mapping any other attributes. This is merely a generalized
     # fallback subsequently replaced by attribute-specific getters.
-    for pep_sign_type_origin in HINT_PEP_SIGNS_TYPE_ORIGIN:
+    for pep_sign_type_origin in HINT_PEP_SIGNS_TYPE_ORIGIN_STDLIB:
         PEP_HINT_SIGN_TO_GET_CAUSE_FUNC[pep_sign_type_origin] = (
             get_cause_or_none_type_origin)
 
