@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -38,7 +38,7 @@ from beartype._decor._code._pep._pephint import pep_code_check_hint
 from beartype._decor._code._pep._pepsnip import (
     PEP_CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER)
 from beartype._decor._data import BeartypeData
-from beartype._decor._typistry import register_typistry_forwardref
+from beartype._decor._cache.cachetype import register_typistry_forwardref
 from beartype._util.cache.utilcacheerror import reraise_exception_cached
 from beartype._util.hint.utilhintget import (
     get_hint_forwardref_classname_relative_to_obj,
@@ -48,11 +48,12 @@ from beartype._util.text.utiltextlabel import (
     label_callable_decorated_param,
     label_callable_decorated_return,
 )
+from beartype._util.text.utiltextmunge import replace_str_substrs
 from collections.abc import Callable, Iterable
 from inspect import Parameter
-from typing import NoReturn, Union
+from typing import NoReturn, Tuple, Union
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ CONSTANTS                         }....................
@@ -92,7 +93,7 @@ def coerce_hint_pep(
       thus invalid as a type hint), raise an exception.
 
     This getter is *not* memoized, due to being only called once per decorated
-    callable parameter or return value.
+    callable parameter or return value and being efficient in any case.
 
     Parameters
     ----------
@@ -156,7 +157,7 @@ def pep_code_check_param(
     hint: object,
     param: Parameter,
     param_index: int,
-) -> 'Tuple[str, bool]':
+) -> Tuple[str, bool]:
     '''
     Python code type-checking the parameter with the passed signature and index
     annotated by a **PEP-compliant type hint** (e.g., :mod:`beartype`-agnostic
@@ -245,11 +246,12 @@ def pep_code_check_param(
         # Generate unmemoized parameter-specific Python code type-checking this
         # exact parameter by globally replacing in this parameter-agnostic
         # code...
-        func_code = func_code.replace(
+        func_code = replace_str_substrs(
+            text=func_code,
             # This placeholder substring cached into this code with...
-            PEP_CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER,
+            old=PEP_CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER,
             # This object representation of this parameter's name.
-            repr(param.name),
+            new=repr(param.name),
         )
 
         # If this code contains one or more relative forward reference
@@ -293,7 +295,7 @@ def pep_code_check_param(
 def pep_code_check_return(
     data: BeartypeData,
     hint: object,
-) -> 'Tuple[str, bool]':
+) -> Tuple[str, bool]:
     '''
     Python code type-checking the return value annotated with a **PEP-compliant
     type hint** (e.g., :mod:`beartype`-agnostic annotation compliant with
@@ -325,7 +327,7 @@ def pep_code_check_return(
 
     # Memoized parameter-agnostic Python code type-checking either a parameter
     # or return value with an arbitrary name.
-    func_code = None
+    func_code: str = None  # type: ignore[assignment]
 
     # True only if type-checking for this return value requires a higher-level
     # caller to prefix the body of this wrapper function with code generating
@@ -386,11 +388,12 @@ def pep_code_check_return(
         # Generate unmemoized parameter-specific Python code type-checking this
         # exact return value by globally replacing in this return-agnostic
         # code...
-        func_code.replace(
+        replace_str_substrs(
+            text=func_code,
             # This placeholder substring cached into this code with...
-            PEP_CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER,
+            old=PEP_CODE_PITH_ROOT_PARAM_NAME_PLACEHOLDER,
             # This object representation of this return value,
-            _RETURN_REPR,
+            new=_RETURN_REPR,
         ),
         # Boolean true only if type-checking this return value requires first
         # localizing a pseudo-random integer.
@@ -439,16 +442,17 @@ def resolve_pep_code_hints_forwardref_class_basename(
     for hint_forwardref_class_basename in hints_forwardref_class_basename:
         # Generate unmemoized callable-specific Python code type-checking this
         # class by globally replacing in this callable-agnostic code...
-        func_code = func_code.replace(
+        func_code = replace_str_substrs(
+            text=func_code,
             # This placeholder substring cached into this code with...
-            (
+            old=(
                 f'{PEP_CODE_HINT_FORWARDREF_UNQUALIFIED_PLACEHOLDER_PREFIX}'
                 f'{hint_forwardref_class_basename}'
                 f'{PEP_CODE_HINT_FORWARDREF_UNQUALIFIED_PLACEHOLDER_SUFFIX}'
             ),
             # Python expression evaluating to this class when accessed
             # via the private "__beartypistry" parameter.
-            register_typistry_forwardref(
+            new=register_typistry_forwardref(
                 # Fully-qualified classname referred to by this forward
                 # reference relative to the decorated callable.
                 get_hint_forwardref_classname_relative_to_obj(
