@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -34,10 +34,15 @@ This private submodule is *not* intended for importation by downstream callers.
 
 # ....................{ IMPORTS                           }....................
 import inspect
-from beartype.cave import CallableTypes
+from beartype.cave import CallableCodeObjectType
+from beartype.roar import BeartypeDecorWrappeeException
+from beartype._util.func.utilfunccodeobj import get_func_codeobj
 from beartype._util.text.utiltextlabel import label_callable_decorated
+from collections.abc import Callable
+from inspect import Signature
+from typing import Optional
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ CLASSES                           }....................
@@ -79,9 +84,14 @@ class BeartypeData(object):
 
     Attributes
     ----------
-    func : CallableTypes
+    func : Callable
         **Decorated callable** (i.e., callable currently being decorated by the
         :func:`beartype.beartype` decorator).
+    func_codeobj = CallableCodeObjectType
+        **Code object** (i.e., instance of the :class:`CodeType` type)
+        underlying the decorated callable.
+    func_sig : inspect.Signature
+        :class:`inspect.Signature` object describing this signature.
 
     Attributes (String)
     ----------
@@ -91,23 +101,20 @@ class BeartypeData(object):
         clashes with existing attributes of the module defining that function,
         this name is obfuscated while still preserving human-readability.
 
-    Attributes (Object)
-    ----------
-    func_sig : inspect.Signature
-        :class:`inspect.Signature` object describing this signature.
-
     .. _PEP 563:
         https://www.python.org/dev/peps/pep-0563
     '''
 
     # ..................{ CLASS VARIABLES                   }..................
-    # Slot *ALL* instance variables defined on this object to minimize space
-    # and time complexity across frequently called @beartype decorations.
+    # Slot all instance variables defined on this object to minimize the time
+    # complexity of both reading and writing variables across frequently
+    # called @beartype decorations. Slotting has been shown to reduce read and
+    # write costs by approximately ~10%, which is non-trivial.
     __slots__ = (
         'func',
+        'func_codeobj',
         'func_sig',
         'func_wrapper_name',
-        '_pep_hint_placeholder_id',
     )
 
     # Coerce instances of this class to be unhashable, preventing spurious
@@ -122,7 +129,7 @@ class BeartypeData(object):
     #
     # See also:
     #     https://docs.python.org/3/reference/datamodel.html#object.__hash__
-    __hash__ = None
+    __hash__ = None  # type: ignore[assignment]
 
     # ..................{ INITIALIZERS                      }..................
     def __init__(self) -> None:
@@ -141,12 +148,13 @@ class BeartypeData(object):
         '''
 
         # Nullify all remaining instance variables.
-        self.func = None
-        self.func_sig = None
-        self.func_wrapper_name = None
+        self.func: Callable = None  # type: ignore[assignment]
+        self.func_codeobj: CallableCodeObjectType = None  # type: ignore[assignment]
+        self.func_sig: Signature = None  # type: ignore[assignment]
+        self.func_wrapper_name: str = None  # type: ignore[assignment]
 
 
-    def reinit(self, func: CallableTypes) -> None:
+    def reinit(self, func: Callable) -> None:
         '''
         Reinitialize this metadata from the passed callable, typically after
         acquisition of a previously cached instance of this class from the
@@ -159,7 +167,7 @@ class BeartypeData(object):
 
         Parameters
         ----------
-        func : CallableTypes
+        func : Callable
             Callable currently being decorated by :func:`beartype.beartype`.
 
         Raises
@@ -168,6 +176,10 @@ class BeartypeData(object):
             If evaluating a postponed annotation on this callable raises an
             exception (e.g., due to that annotation referring to local state no
             longer accessible from this deferred evaluation).
+        BeartypeDecorWrappeeException
+            If this callable is neither a pure-Python function *nor* method;
+            equivalently, if this callable is either C-based *or* a class or
+            object defining the ``__call__()`` dunder method.
 
         .. _PEP 563:
            https://www.python.org/dev/peps/pep-0563
@@ -180,12 +192,17 @@ class BeartypeData(object):
         # Callable currently being decorated.
         self.func = func
 
+        # Code object underlying this callable if this callable is a
+        # pure-Python function or method *OR* raise an exception otherwise.
+        self.func_codeobj = get_func_codeobj(
+            func=func, exception_cls=BeartypeDecorWrappeeException)
+
         # Machine-readable name of the wrapper function to be generated.
         self.func_wrapper_name = f'__beartyped_{func.__name__}'
 
         # Nullify all remaining attributes for safety *BEFORE* passing this
         # object to any functions (e.g., resolve_hints_postponed_if_needed()).
-        self.func_sig = None
+        self.func_sig = None  # type: ignore[assignment]
 
         # Resolve all postponed annotations if any on this callable *BEFORE*
         # parsing the actual annotations these postponed annotations refer to.
