@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -80,6 +80,17 @@ This private submodule is *not* intended for importation by downstream callers.
 #      format_await = 'await ' if inspect.iscoroutinefunction(func) else ''
 #* Oh, and note that our defined wrapper function must also be preceded by the
 #  "async " keyword. So, we'll also need to augment "CODE_SIGNATURE".
+#FIXME: As a counterargument to the above approach, note this commentary I
+#stumbled across while researching an entirely separate topic:
+#    "...trying to automatically detect whether a function is sync or async
+#    it’s almost always a bad idea, because it’s very difficult to do reliably.
+#    Instead it’s almost always better to make the user say explicitly which
+#    one they mean, for example by having two versions of a decorator and
+#    telling the user to use @mydecorator_sync on sync functions and
+#    @mydecorator_async on async functions."
+#Is this actually the case? Clearly, we'll need to research just how
+#deterministic the inspect.isawaitable() tester is. Does that tester fall down
+#(i.e., return false negatives or positives) in well-known edge cases?
 #FIXME: Unit test this extensively, please.
 
 #FIXME: Non-critical optimization: if the active Python interpreter is already
@@ -126,6 +137,91 @@ This private submodule is *not* intended for importation by downstream callers.
 #  "_PARAM_KIND_IGNORABLE" set.
 #* Remove the "_PARAM_KIND_IGNORABLE" set entirely.
 
+#FIXME: [NEW PROJECT] Consider creating a new "beartype/bearclaw" project *OR
+#SOMETHING* (e.g., "bearcall", "bearely", etc.) to enable arbitrary O(1)
+#runtime type checking. Actually, "bearely" is a super-hot name, so let's run
+#with that, shall we? By "arbitrary," we mean just that: O(1) runtime type
+#checking that anyone can perform in any arbitrary expression without having to
+#isolate that checking to a callable signature.
+#
+#First, let's spec the public API. Fortunately, that's trivial. Just as with
+#"beartype", we define only a single public turbo-charged function:
+#* Define a public "bearely" package.
+#* Define a private "bearely._main" submodule *OR SOMETHING.*
+#* In that submodule:
+#  * Define a public istypedas() tester with the signature:
+#
+#    def istypedas(obj: object, hint: object) -> bool:
+#
+#    ...where "obj" is any arbitrary object and "hint" is any PEP-compliant
+#    type hint (or, more generally, any @beartype-compliant type hint).
+#  * Actually, define a public is_typed_as() alias to the istypedas() tester.
+#    Not everyone wants cute names; PEP 8-compliant snake case is often
+#    preferable and we ourselves would probably prefer the former, for example.
+#* Define a dunder "bearely.__init__" submodule publicizing
+#  bearely._main.istypedas() as "bearely.istypedas().
+#* Have "bearely" depend upon "beartype" as its only mandatory runtime
+#  dependency.
+#
+#*YUP.* istypedas() is the single public turbo-charged function declared by
+#this package. The nomenclature for this tester derives, of course, from the
+#builtin isinstanceof() and issubclassof() builtins. istypedas() could be
+#considered a generalization or proper superset of both -- in that anything you
+#can do with those builtins you can do with istypedas(), but you can also do
+#*MUCH* more with istypedas().
+#
+#Fortuitously, implementing istypedas() in terms of the existing @beartype
+#decorator is trivial and requires absolutely *NO* refactoring of the
+#"beartype" codebase itself, which is certainly nice (albeit non-essential):
+#* Internally, istypedas() should maintain a *non-LRU* cache (probably in a
+#  separate "bearely._cache" submodule as a simple dictionary) named
+#  "HINT_OR_HINT_REPR_TO_BEARTYPE_WRAPPER" mapping from each arbitrary
+#  PEP-compliant type hint (passed as the second parameter to istypedas()) to
+#  the corresponding wrapper function dynamically generated by the @beartype
+#  decorator checking an arbitrary object against that hint. However, note
+#  there there's a significant caveat here:
+#  * *NOT ALL HINTS ARE CACHABLE.* If the passed hint is *NOT* cachable, we
+#    should instead cache that hint under its machine-readable repr() string.
+#    While slower to generate, generating that string is still guaranteed to be
+#    *MUCH* faster than dynamically declaring a new function each call.
+#* The trivial way to implement the prior item is to dynamically define one new
+#  private @beartype-decorated noop function accepting an arbitrary parameter
+#  type-hinted by each type hint: e.g.,
+#      # Pseudo-code, obviously. Again, this snippet should probably be
+#      # shifted into a new "bearely._snip" submodule.
+#      is_typed_as_wrapper = exec(f'''
+#      @beartype
+#      def is_typed_as_wrapper(obj: {hint}): pass
+#      ''')
+#* After either defining and caching that wrapper into the above dictionary
+#  *OR* retrieved a previously wrapper from that dictionary, trivially
+#  implement this check with EAFP as follows:
+#      try:
+#          is_typed_as_wrapper(obj)
+#          return True
+#      except:
+#          return False
+#
+#*DONE.* Sweet, yah? The above can and should be heavily optimized, of course.
+#How? That remains to be determined. The principle issue with the above
+#approach is that it unnecessarily incurs an additional stack frame. Since the
+#original is_typed_as_wrapper() function wrapped by @beartype doesn't actually
+#do anything, it would be really nice if the wrapper generated by @beartype
+#omitted the call to that original function.
+#
+#This might be easier than expected. You're probably thinking AST inspector or
+#disassembly, right? Neither of those two things are easy or fast, so let's do
+#neither. Is there any alternative? There might be. In theory, the code object
+#for any callable whose implementation is literally "pass" should be trivially
+#detectable via metadata on that object. If nothing else, the byte code for
+#that object should be a constant size; any code object whose byte code is
+#larger than that size is *NOT* a "pass" noop.
+#
+#In any case, @beartype should efficiently detect noop callables and avoid
+#calling those callables from the wrapper functions it generates for those
+#callables. This would be genuinely useful from the general-purpose
+#perspective, which means we should make this happen.
+
 # ....................{ IMPORTS                           }....................
 import functools, random
 from beartype.roar import (
@@ -134,19 +230,19 @@ from beartype.roar import (
 )
 from beartype._decor._code.codemain import generate_code
 from beartype._decor._code.codesnip import (
-    PARAM_NAME_FUNC, PARAM_NAME_TYPISTRY)
+    ARG_NAME_FUNC, ARG_NAME_TYPISTRY)
 from beartype._decor._data import BeartypeData
-from beartype._decor._typistry import bear_typistry
+from beartype._decor._cache.cachetype import bear_typistry
 from beartype._util.cache.pool.utilcachepoolobjecttyped import (
     acquire_object_typed, release_object_typed)
 from beartype._decor._code._pep._error.peperror import (
     raise_pep_call_exception)
 from beartype._util.text.utiltextmunge import number_lines
-from typing import TYPE_CHECKING
+from typing import Callable, TYPE_CHECKING
 # from beartype._util.utilobject import get_object_name
 # from types import FunctionType
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ CONSTANTS                         }....................
@@ -175,11 +271,12 @@ as a private default parameter to the signatures of these functions.
 '''
 
 # ....................{ DECORATORS                        }....................
-def beartype(func):
+def beartype(func: Callable) -> Callable:
     '''
-    Decorate the passed **callable** (e.g., function, method) to validate both
-    all annotated parameters passed to this callable *and* the annotated value
-    returned by this callable if any.
+    Decorate the passed **pure-Python callable** (e.g., function or method
+    declared in Python rather than C) to validate both all annotated parameters
+    passed to this callable *and* the annotated value returned by this callable
+    if any.
 
     This decorator performs rudimentary type checking based on Python 3.x
     function annotations, as officially documented by PEP 484 ("Type Hints").
@@ -195,14 +292,14 @@ def beartype(func):
 
     Parameters
     ----------
-    func : CallableTypes
+    func : Callable
         **Non-class callable** (i.e., callable object that is *not* a class) to
         be decorated by a dynamically generated new callable wrapping this
         original callable with pure-Python type-checking.
 
     Returns
     ----------
-    CallableTypes
+    Callable
         Dynamically generated new callable wrapping this original callable with
         pure-Python type-checking.
 
@@ -224,16 +321,20 @@ def beartype(func):
             fully-qualified classnames).
           * **Tuple unions** (i.e., tuples containing one or more classes
             and/or forward references).
-    BeartypeDecorParamNameException
-        If the name of any parameter declared on this callable is prefixed by
-        the reserved substring ``__beartype_``.
     BeartypeDecorHintPep563Exception
         If `PEP 563`_ is active for this callable and evaluating a **postponed
         annotation** (i.e., annotation whose value is a string) on this
         callable raises an exception (e.g., due to that annotation referring to
         local state no longer accessible from this deferred evaluation).
+    BeartypeDecorParamNameException
+        If the name of any parameter declared on this callable is prefixed by
+        the reserved substring ``__beartype_``.
     BeartypeDecorWrappeeException
-        If this callable is either uncallable or a class.
+        If this callable is either:
+
+        * Uncallable.
+        * A class, which :mod:`beartype` currently fails to support.
+        * A C-based callable (e.g., builtin, third-party C extension).
 
     .. _PEP 484:
        https://www.python.org/dev/peps/pep-0484
@@ -313,10 +414,11 @@ def beartype(func):
     # For the above reasons, the *ONLY* attribute that should be passed is the
     # wrapper-specific "__beartype_func" attribute.
     local_attrs = {
-        PARAM_NAME_FUNC: func,
-        PARAM_NAME_TYPISTRY: bear_typistry,
+        ARG_NAME_FUNC: func,
+        ARG_NAME_TYPISTRY: bear_typistry,
     }
 
+    #FIXME: Uncomment after uncommenting the corresponding logic below.
     # Fully-qualified name of this undecorated callable to be decorated.
     # func_name_qualified = get_object_name(func)
 
@@ -353,8 +455,14 @@ def beartype(func):
     #FIXME: Actually, we absolutely *DO* want to leverage the example
     #documented below of leveraging the compile() builtin. We want to do so
     #explicitly to pass something other than "<string>" here -- ideally,
-    #"func.__code__.co_filename", ensuring that this wrapper function
-    #shares the same absolute filename as that of the original function.
+    #"func.__code__.co_filename", ensuring that this wrapper function shares
+    #the same absolute filename as that of the original function. To do so:
+    #
+    #* Implement the
+    #  beartype._util.utilcallable.get_callable_filename_or_placeholder()
+    #  getter.
+    #* Call that function here to obtain that filename.
+    #
     #Note that a similar example (also leveraging the exec() builtin, which
     #frankly seems excessive) is also given by:
     #    https://stackoverflow.com/a/42478041/2809027
@@ -436,12 +544,12 @@ def beartype(func):
     # dictionary is guaranteed to contain a key with this wrapper's name whose
     # value is this wrapper. Ergo, no additional validation of the existence of
     # this key or type of this wrapper is needed.
-    func_wrapper = local_attrs[func_data.func_wrapper_name]
+    func_wrapper: Callable = local_attrs[func_data.func_wrapper_name]  # type: ignore[assignment]
 
     # Declare this wrapper to be generated by @beartype, which tests for the
     # existence of this attribute above to avoid re-decorating callables
     # already decorated by @beartype by efficiently reducing to a noop.
-    func_wrapper.__beartype_wrapper = True
+    func_wrapper.__beartype_wrapper = True  # type: ignore[attr-defined]
 
     # Propagate identifying metadata (stored as special attributes) from the
     # original function to this wrapper for debuggability, including:
@@ -482,7 +590,7 @@ if (
 #         return
 #
 # Tragically, Python fails to support module-scoped "return" statements. *sigh*
-    def beartype(func):
+    def beartype(func: Callable) -> Callable:
         '''
         Identity decorator.
 
