@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -11,39 +11,12 @@ the same list are typically of the same type).
 This private submodule is *not* intended for importation by downstream callers.
 '''
 
-# ....................{ TODO                              }....................
-#FIXME: Consider improving the safety of the "KeyPool" class. The current
-#implementation is overly naive. For both simplicity and efficiency, we avoid
-#explicitly validating various conditions on passed objects. For example, the
-#object passed to the KeyPool.release() method is assumed -- but *NOT*
-#validated -- to have been previously returned from a call to the
-#KeyPool.acquire() method.
-#
-#If this ever becomes a demonstrable issue, validation can be implemented with
-#only mild space and time costs as follows:
-#
-#* Define a new "KeyPool._pool_item_ids_acquired" set instance variable,
-#  containing the object IDs of all pool items returned by prior calls to the
-#  acquire() method that have yet to be passed to the release() method.
-#* Define a new "KeyPool._pool_item_ids_released" set instance variable,
-#  containing the object IDs of all pool items passed to prior calls to the
-#  release() method that have yet to be returned from the acquire() method.
-#* Refactor the acquire() method to (in order):
-#  * Validate that the pool item to be returned is in the
-#    "_pool_item_ids_released" set but *NOT* the "_pool_item_ids_acquired" set.
-#  * Remove this item from the "_pool_item_ids_released" set.
-#  * Add this item from the "_pool_item_ids_acquired" set.
-#  * Note that the above procedure fails to account for the edge case of newly
-#    created pool items, which should reside in *NEITHER* set. </sigh>
-#* Refactor the release() method to (in order):
-#  * Validate that the passed pool item is in the
-#    "_pool_item_ids_acquired" set but *NOT* the "_pool_item_ids_released" set.
-#  * Remove this item from the "_pool_item_ids_acquired" set.
-#  * Add this item from the "_pool_item_ids_released" set.
-
 # ....................{ IMPORTS                           }....................
+from beartype.roar import _BeartypeUtilCachedKeyPoolException
 from collections import defaultdict
+from collections.abc import Callable, Hashable
 from threading import Lock
+from typing import Any, Dict, Union
 
 # ....................{ CLASSES                           }....................
 class KeyPool(object):
@@ -57,13 +30,23 @@ class KeyPool(object):
 
     Attributes
     ----------
-    _key_to_pool : dict
-        Dictionary mapping from **arbitrary keys** (i.e., hashable objects) to
-        lists of zero or more arbitrary objects cached under those keys. For
-        both efficiency and simplicity, this dictionary is defined as a
-        :class:`defaultdict` implicitly initializing missing keys on initial
-        access to the empty list.
-    _pool_item_maker : CallableTypes
+    _key_to_pool : defaultdict
+        Dictionary mapping from an **arbitrary key** (i.e., hashable object) to
+        corresponding **pool** (i.e., list of zero or more arbitrary objects
+        referred to as "pool items" cached under that key). For both efficiency
+        and simplicity, this dictionary is defined as a :class:`defaultdict`
+        implicitly initializing missing keys on initial access to the empty
+        list.
+    _pool_item_id_to_is_acquired : dict
+        Dictionary mapping from the unique object identifier of a **pool item**
+        (i.e., arbitrary object cached under a pool of the :attr:`_key_to_pool`
+        ditionary) to a boolean that is either:
+
+        * `True` if that item is currently **acquired** (i.e., most recently
+          returned by a call to the :meth:`acquire` method).
+        * `False` if that item is currently **released** (i.e., most recently
+          passed to a call to the :meth:`release` method).
+    _pool_item_maker : Callable
         Caller-defined factory callable internally called by the
         :meth:`acquire` method on attempting to acquire a non-existent object
         from an **empty pool. See :meth:`__init__` for further details.
@@ -79,18 +62,28 @@ class KeyPool(object):
     '''
 
     # ..................{ CLASS VARIABLES                   }..................
-    # Slot *ALL* instance variables defined on this object to minimize space
-    # and time complexity across frequently called @beartype decorations.
-    __slots__ = ('_key_to_pool', '_pool_item_maker', '_thread_lock')
+    # Slot all instance variables defined on this object to minimize the time
+    # complexity of both reading and writing variables across frequently called
+    # @beartype decorations. Slotting has been shown to reduce read and write
+    # costs by approximately ~10%, which is non-trivial.
+    __slots__ = (
+        '_key_to_pool',
+        '_pool_item_id_to_is_acquired',
+        '_pool_item_maker',
+        '_thread_lock',
+    )
 
     # ..................{ INITIALIZER                       }..................
-    def __init__(self, item_maker: 'CallableTypes') -> None:
+    def __init__(
+        self,
+        item_maker: Union[type, Callable],
+    ) -> None:
         '''
         Initialize this key pool with the passed factory callable.
 
         Parameters
         ----------
-        item_maker : CallableTypes
+        item_maker : Union[type, Callable[[Hashable,], Any]]
             Caller-defined factory callable internally called by the
             :meth:`acquire` method on attempting to acquire a non-existent
             object from an **empty pool** (i.e., either a missing key *or* an
@@ -103,11 +96,10 @@ class KeyPool(object):
 
             .. code-block:: python
 
-               from beartype.cave import HashableType
-               def item_maker(key: HashableType) -> object: ...
+               from collections.abc import Hashable
+               def item_maker(key: Hashable) -> object: ...
         '''
-        assert callable(item_maker), (
-            '{!r} not callable.'.format(item_maker))
+        assert callable(item_maker), f'{repr(item_maker)} not callable.'
 
         # Classify these parameters as instance variables.
         self._pool_item_maker = item_maker
@@ -122,11 +114,12 @@ class KeyPool(object):
         #     >>> dd = defaultdict(default_factory=list)
         #     >>> dd['ee']
         #     KeyError: 'ee'
-        self._key_to_pool = defaultdict(list)
+        self._key_to_pool: Dict[Hashable, list] = defaultdict(list)
+        self._pool_item_id_to_is_acquired: Dict[int, bool] = {}
         self._thread_lock = Lock()
 
     # ..................{ METHODS                           }..................
-    def acquire(self, key: 'HashableType' = None) -> object:
+    def acquire(self, key: Hashable = None) -> object:
         '''
         Acquire an arbitrary object associated with the passed **arbitrary
         key** (i.e., hashable object).
@@ -137,11 +130,12 @@ class KeyPool(object):
         exists no such list or such a list exists but is empty), this method
         effectively (in order):
 
-        #. If no such list exists, creates a new empty list associated with
+        #. If no such list exists, create a new empty list associated with
            this key.
-        #. Creates a new object to be returned by calling the user-defined
+        #. Create a new object to be returned by calling the user-defined
            :meth:`_pool_item_maker` factory callable.
-        #. Appends this object to this list.
+        #. Append this object to this list.
+        #. Add/Update acquisition state of the object to True
         #. Returns this object.
 
         Parameters
@@ -174,23 +168,29 @@ class KeyPool(object):
             # efficient than our explicitly validating this constraint.
             pool = self._key_to_pool[key]
 
-            # Return either...
-            return (
-                # A new object associated with this key...
-                self._pool_item_maker(key)
-                # If the list associated with this key is empty (i.e., this
-                # method has been called more frequently than the corresponding
-                # release() method for this key)...
-                if not pool
-                # Else, the list associated with this key is non-empty (i.e.,
-                # this method has been called less frequently than the
+            # Pool item associated with this key, defined as either...
+            pool_item = (
+                # The last item popped (i.e., removed) from this list...
+                pool.pop()
+                # If the list associated with this key is non-empty (i.e., this
+                # method has been called less frequently than the corresponding
+                # release() method for this key);
+                if pool else
+                # Else, the list associated with this key is empty (i.e.,
+                # this method has been called more frequently than the
                 # corresponding release() method for this key). In this case,
-                # pop (i.e., remove) and return the last item from this list.
-                else pool.pop()
+                # an arbitrary object associated with this key.
+                self._pool_item_maker(key)
             )
 
+            # Record this item to have now been acquired.
+            self._pool_item_id_to_is_acquired[id(pool_item)] = True
 
-    def release(self, item: object, key: 'HashableType' = None) -> None:
+            # Return this item.
+            return pool_item
+
+
+    def release(self, item: object, key: Hashable = None) -> None:
         '''
         Release the passed object acquired by a prior call to the
         :meth:`acquire` method passed the same passed **arbitrary key** (i.e.,
@@ -212,12 +212,25 @@ class KeyPool(object):
         Raises
         ----------
         TypeError
-            If this key is unhashable and thus *not* a key.
+            If this key is unhashable (i.e. *not* a key).
+        _BeartypeUtilCachedKeyPoolException
+            If this pool item was *not* acquired (i.e., returned by a prior
+            call to the :meth:`acquire` method) and thus ineligible for
+            release.
         '''
 
         # In a thread-safe manner...
-        #
-        # The best things in life are free or two-liners. This is the latter.
         with self._thread_lock:
-            # Append this object to the list associated with this key.
+            # Integer uniquely identifying this previously acquired pool item.
+            item_id = id(item)
+
+            # If this item was *NOT* previously acquired, raise an exception.
+            if not self._pool_item_id_to_is_acquired.get(item_id, False):
+                raise _BeartypeUtilCachedKeyPoolException(
+                    f'Key pool unacquired item {repr(item)} not releasable.')
+
+            # Record this item to have now been released.
+            self._pool_item_id_to_is_acquired[item_id] = False
+
+            # Append this item to the pool associated with this key.
             self._key_to_pool[key].append(item)
