@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -25,14 +25,14 @@ from beartype._util.hint.data.pep.proposal.utilhintdatapep484 import (
 )
 from beartype._util.py.utilpyversion import IS_PYTHON_AT_LEAST_3_7
 from beartype._util.utilobject import is_object_subclass
-from typing import Generic, NewType
+from typing import Any, Generic, NewType, Optional
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ TESTERS ~ ignorable               }....................
 def is_hint_pep484_ignorable_or_none(
-    hint: object, hint_sign: object) -> 'Optional[bool]':
+    hint: object, hint_sign: object) -> Optional[bool]:
     '''
     ``True`` only if the passed object is a `PEP 484`_-compliant **ignorable
     type hint,** ``False`` only if this object is a `PEP 484`_-compliant
@@ -127,7 +127,7 @@ def is_hint_pep484_ignorable_or_none(
         # "typing.Generic[T]"), return true.
         #
         # Note that we intentionally avoid calling the
-        # get_hint_pep_type_origin_or_none() function here, which has been
+        # get_hint_pep_stdlib_type_or_none() function here, which has been
         # intentionally designed to exclude PEP-compliant type hints
         # originating from "typing" type origins for stability reasons.
         if getattr(hint, '__origin__', None) is Generic:
@@ -206,6 +206,19 @@ def is_hint_pep484_forwardref(hint: object) -> bool:
 if IS_PYTHON_AT_LEAST_3_7:
     def is_hint_pep484_generic(hint: object) -> bool:
 
+        # Avoid circular import dependencies.
+        from beartype._util.hint.pep.utilhintpepget import (
+            get_hint_pep_generic_type_or_none)
+
+        # If this hint is *NOT* a class, this hint is *NOT* an unsubscripted
+        # generic but could still be a subscripted generic (i.e., generic
+        # subscripted by one or more PEP-compliant child type hints). To
+        # decide, reduce this hint to the object originating this hint if any,
+        # enabling the subsequent test to test whether this origin object is an
+        # unsubscripted generic, which would then imply this hint to be a
+        # subscripted generic. If this strikes you as insane, you're not alone.
+        hint = get_hint_pep_generic_type_or_none(hint)
+
         # Return true only if this hint is a subclass of the "typing.Generic"
         # abstract base class (ABC), in which case this hint is a user-defined
         # generic.
@@ -239,23 +252,31 @@ if IS_PYTHON_AT_LEAST_3_7:
         # passed objects that *CANNOT* be guaranteed to be PEP-compliant.
         # Indeed, the high-level is_hint_pep() tester establishing the
         # PEP-compliance of arbitrary objects internally calls this lower-level
-        # tester to do so. Since the get_hint_pep484_generic_bases_unerased_or_none() getter
-        # internally reduces to returning the tuple of the general-purpose
+        # tester to do so. Since the
+        # get_hint_pep484_generic_bases_unerased_or_none() getter internally
+        # reduces to returning the tuple of the general-purpose
         # "__orig_bases__" dunder attribute formalized by PEP 560, testing
         # whether that tuple is non-empty or not in no way guarantees this
         # object to be a PEP-compliant generic.
-        return is_object_subclass(hint, Generic)
-# Else, the active Python interpreter targets Python 3.6. In this case,
+        return is_object_subclass(hint, Generic)  # type: ignore[arg-type]
+
+# Else, the active Python interpreter targets Python 3.6.x. In this case,
 # implement this function specific to this Python version.
 else:
-    # Import the Python < 3.7.0-specific metaclass required by this tester.
-    from typing import GenericMeta
+    # Import the Python 3.6.x-specific metaclass required by this tester.
+    from typing import GenericMeta  # type: ignore[attr-defined]
 
     def is_hint_pep484_generic(hint: object) -> bool:
 
         # Avoid circular import dependencies.
+        from beartype._util.hint.pep.utilhintpepget import (
+            get_hint_pep_generic_type_or_none)
         from beartype._util.hint.pep.utilhintpeptest import (
-            is_hint_pep_class_typing)
+            is_hint_pep_type_typing)
+
+        # If this hint is *NOT* a class, reduce this hint to the object
+        # originating this hint if any. See the above tester for details.
+        hint = get_hint_pep_generic_type_or_none(hint)
 
         # Return true only if this hint is a subclass *NOT* defined by the
         # "typing" module whose class is the "typing.GenericMeta" metaclass, in
@@ -282,7 +303,7 @@ else:
         #   disingenuous "typing" pseudo-generics.
         return (
             isinstance(hint, GenericMeta) and
-            not is_hint_pep_class_typing(hint)
+            not is_hint_pep_type_typing(hint)
         )
 
 
@@ -411,7 +432,7 @@ def is_hint_pep484_newtype(hint: object) -> bool:
 
 # ....................{ GETTERS ~ forwardref              }....................
 @callable_cached
-def get_hint_pep484_forwardref_class_basename(hint: object) -> str:
+def get_hint_pep484_forwardref_class_basename(hint: Any) -> str:
     '''
     **Unqualified classname** (i.e., name of a class *not* containing a ``.``
     delimiter and thus relative to the fully-qualified name of the submodule
@@ -468,7 +489,7 @@ def get_hint_pep484_forwardref_class_basename(hint: object) -> str:
     return hint.__forward_arg__
 
 # ....................{ GETTERS ~ newtype                 }....................
-def get_hint_pep484_newtype_class(hint: object) -> type:
+def get_hint_pep484_newtype_class(hint: Any) -> type:
     '''
     User-defined class aliased by the passed `PEP 484`_-compliant **new type**
     (i.e., closure created and returned by the :func:`typing.NewType` closure
@@ -517,7 +538,7 @@ def get_hint_pep484_newtype_class(hint: object) -> type:
 
 # ....................{ GETTERS ~ generic                 }....................
 @callable_cached
-def get_hint_pep484_generic_base_erased_from_unerased(hint: object) -> type:
+def get_hint_pep484_generic_base_erased_from_unerased(hint: Any) -> type:
     '''
     Erased superclass originating the passed `PEP 484`_-compliant **unerased
     pseudo-superclass** (i.e., :mod:`typing` object originally listed as a
@@ -563,7 +584,7 @@ def get_hint_pep484_generic_base_erased_from_unerased(hint: object) -> type:
 
 
 @callable_cached
-def get_hint_pep484_generic_bases_unerased(hint: object) -> 'Tuple[object]':
+def get_hint_pep484_generic_bases_unerased(hint: Any) -> tuple:
     '''
     Tuple of all unerased :mod:`typing` **pseudo-superclasses** (i.e.,
     :mod:`typing` objects originally listed as superclasses prior to their
@@ -674,7 +695,7 @@ def get_hint_pep484_generic_bases_unerased(hint: object) -> 'Tuple[object]':
     #        hint_base = hint_orig_mro[hint_orig_mro_index_curr]
     #
     #        # If this superclass is a typing attribute...
-    #        if is_hint_pep_class_typing(hint_base):
+    #        if is_hint_pep_type_typing(hint_base):
     #            # Avoid inserting this attribute into the "hint_orig_mro" list.
     #            # Most typing attributes are *NOT* actual classes and those that
     #            # are have no meaningful public superclass. Ergo, iteration
@@ -741,11 +762,20 @@ def get_hint_pep484_generic_bases_unerased(hint: object) -> 'Tuple[object]':
     #  * Return a frozenset of this list, thus implicitly eliminating
     #    duplicate superclasses.
 
+    # Avoid circular import dependencies.
+    from beartype._util.hint.pep.utilhintpepget import (
+        get_hint_pep_generic_type_or_none)
+
+    # If this hint is *NOT* a class, reduce this hint to the object originating
+    # this hint if any. See is_hint_pep484_generic() tester for details.
+    hint = get_hint_pep_generic_type_or_none(hint)
+
     # If this hint is *NOT* a PEP 484-compliant generic, raise an exception.
     if not is_hint_pep484_generic(hint):
         raise BeartypeDecorHintPep484Exception(
-            f'PEP type hint "{repr(hint)}" neither '
-            f'PEP 484 generic nor PEP 544 protocol.')
+            f'Type hint {repr(hint)} neither '
+            f'PEP 484 generic nor PEP 544 protocol.'
+        )
     # Else, this hint is a PEP 484-compliant generic.
 
     # Unerased pseudo-superclasses of this generic if any *OR* "None"
