@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -13,12 +13,13 @@ This private submodule is *not* intended for importation by downstream callers.
 '''
 
 # ....................{ IMPORTS                           }....................
-from beartype.cave import HintPep585Type
+from beartype.cave import HintGenericSubscriptedType
 from beartype.roar import BeartypeDecorHintPep585Exception
 from beartype._util.cache.utilcachecall import callable_cached
 from beartype._util.py.utilpyversion import IS_PYTHON_AT_LEAST_3_9
+from typing import Any, Set, Tuple
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ VALIDATORS                        }....................
@@ -45,25 +46,50 @@ def die_unless_hint_pep585_generic(hint: object) -> None:
     # If this hint is *NOT* a PEP 585-compliant generic, raise an exception
     if not is_hint_pep585_generic(hint):
         raise BeartypeDecorHintPep585Exception(
-            f'PEP type hint "{repr(hint)}" not PEP 585 generic.')
+            f'Type hint {repr(hint)} not PEP 585 generic.')
 
 # ....................{ TESTERS                           }....................
 # If the active Python interpreter targets at least Python >= 3.9 and thus
 # supports PEP 585, correctly declare this function.
 if IS_PYTHON_AT_LEAST_3_9:
-    def is_hint_pep585(hint: object) -> bool:
-        return isinstance(hint, HintPep585Type)
+    def is_hint_pep585_builtin(hint: object) -> bool:
+
+        # Avoid circular import dependencies.
+        from beartype._util.hint.pep.utilhintpeptest import is_hint_pep_generic
+
+        # Return true only if this hint...
+        return (
+            # Is either a PEP 484- or -585-compliant subscripted generic or
+            # PEP 585-compliant builtin *AND*...
+            isinstance(hint, HintGenericSubscriptedType) and
+            # Is *NOT* a PEP 484- or -585-compliant subscripted generic.
+            not is_hint_pep_generic(hint)
+        )
 
 
     @callable_cached
     def is_hint_pep585_generic(hint: object) -> bool:
 
+        # Avoid circular import dependencies.
+        from beartype._util.hint.pep.utilhintpepget import (
+            get_hint_pep_generic_type_or_none)
+
+        # If this hint is *NOT* a class, reduce this hint to the object
+        # originating this hint if any. See the comparable
+        # is_hint_pep484_generic() tester for further details.
+        hint = get_hint_pep_generic_type_or_none(hint)
+
+        # Tuple of all pseudo-superclasses originally subclassed by the passed
+        # hint if this hint is a generic *OR* false otherwise.
+        hint_bases_erased = getattr(hint, '__orig_bases__', False)
+
         # Unsurprisingly, PEP 585-compliant generics have absolutely *NO*
         # commonality with PEP 484-compliant generics. While the latter are
         # trivially detectable as subclassing "typing.Generic" after type
         # erasure, the former are *NOT*. The only means of deterministically
         # deciding whether or not any given class is a PEP 585-compliant
         # generic is as follows:
+        #
         # * That class defines both the __class_getitem__() dunder method *AND*
         #   the "__orig_bases__" instance variable. Note that this condition in
         #   and of itself is insufficient to decide PEP 585-compliance as a
@@ -75,31 +101,24 @@ if IS_PYTHON_AT_LEAST_3_9:
         #   objects.
         #
         # Return true only if this hint satisfies *ALL* of the above conditions.
-        # Specifically, return true only if...
-        return (
-            # This hint defines the "__orig_bases__" dunder instance variable
-            # listing all pseudo-superclasses originally subclassed by this
-            # hint *AND*...
-            #
-            # Note that we could technically also test that this hint defines
-            # the __class_getitem__() dunder method. Since the subsequent
-            # condition suffices to ensure that this hint is a PEP
-            # 585-compliant generic, however, there exists little benefit to
-            # doing so.
-            hasattr(hint, '__orig_bases__') and
-            # At least one pseudo-superclass originally subclassed by this hint
-            # is a PEP 585-compliant type hint.
-            any(
-                is_hint_pep585(hint_base_erased)
-                for hint_base_erased in hint.__orig_bases__
-            )
+        # Specifically, return true only if one or more pseudo-superclasses
+        # originally subclassed by this hint are themselves PEP 585-compliant
+        # type hints.
+        #
+        # Note we could technically also test that this hint defines the
+        # __class_getitem__() dunder method. Since this condition suffices to
+        # ensure that this hint is a PEP 585-compliant generic, however, there
+        # exists little benefit to doing so.
+        return hint_bases_erased and any(
+            is_hint_pep585_builtin(hint_base_erased)
+            for hint_base_erased in hint_bases_erased
         )
 
 # Else, the active Python interpreter targets at most Python < 3.9 and thus
 # fails to support PEP 585. In this case, fallback to declaring this function
 # to unconditionally return False.
 else:
-    def is_hint_pep585(hint: object) -> bool:
+    def is_hint_pep585_builtin(hint: object) -> bool:
         return False
 
     def is_hint_pep585_generic(hint: object) -> bool:
@@ -108,12 +127,12 @@ else:
 # ....................{ TESTERS ~ doc                     }....................
 # Docstring for this function regardless of implementation details.
 # Docstring for this function regardless of implementation details.
-is_hint_pep585.__doc__ = '''
-    ``True`` only if the passed object is a C-based `PEP 585`_-compliant **type
-    hint** (i.e., C-based type hint instantiated by subscripting either a
-    concrete builtin container class like :class:`list` or :class:`tuple` *or*
-    an abstract base class (ABC) declared by the :mod:`collections.abc`
-    submodule like :class:`collections.abc.Iterable` or
+is_hint_pep585_builtin.__doc__ = '''
+    ``True`` only if the passed object is a C-based `PEP 585`_-compliant
+    **builtin type hint** (i.e., C-based type hint instantiated by subscripting
+    either a concrete builtin container class like :class:`list` or
+    :class:`tuple` *or* an abstract base class (ABC) declared by the
+    :mod:`collections.abc` submodule like :class:`collections.abc.Iterable` or
     :class:`collections.abc.Sequence`).
 
     This tester is intentionally *not* memoized (e.g., by the
@@ -166,14 +185,14 @@ is_hint_pep585_generic.__doc__ = '''
     Returns
     ----------
     bool
-        ``True`` only if this object is a generic.
+        ``True`` only if this object is a `PEP 585`_-compliant generic.
 
     .. _PEP 585:
        https://www.python.org/dev/peps/pep-0585
     '''
 
 # ....................{ GETTERS                           }....................
-def get_hint_pep585_generic_bases_unerased(hint: object) -> 'Tuple[object]':
+def get_hint_pep585_generic_bases_unerased(hint: Any) -> tuple:
     '''
     Tuple of all unerased `PEP 585`_-compliant **pseudo-superclasses** (i.e.,
     :mod:`typing` objects originally listed as superclasses prior to their
@@ -210,7 +229,15 @@ def get_hint_pep585_generic_bases_unerased(hint: object) -> 'Tuple[object]':
        https://www.python.org/dev/peps/pep-0585
     '''
 
-    # If this hint is *NOT* a PEP 585-compliant generic, raise an exception
+    # Avoid circular import dependencies.
+    from beartype._util.hint.pep.utilhintpepget import (
+        get_hint_pep_generic_type_or_none)
+
+    # If this hint is *NOT* a class, reduce this hint to the object originating
+    # this hint if any. See the is_hint_pep484_generic() tester for details.
+    hint = get_hint_pep_generic_type_or_none(hint)
+
+    # If this hint is *NOT* a PEP 585-compliant generic, raise an exception.
     die_unless_hint_pep585_generic(hint)
 
     # Return the tuple of all unerased pseudo-superclasses of this generic.
@@ -221,7 +248,7 @@ def get_hint_pep585_generic_bases_unerased(hint: object) -> 'Tuple[object]':
 
 
 @callable_cached
-def get_hint_pep585_generic_typevars(hint: object) -> 'Tuple[TypeVar]':
+def get_hint_pep585_generic_typevars(hint: object) -> Tuple[type, ...]:
     '''
     Tuple of all **unique type variables** (i.e., subscripted :class:`TypeVar`
     instances of the passed `PEP 585`_-compliant generic listed by the caller
@@ -247,7 +274,7 @@ def get_hint_pep585_generic_typevars(hint: object) -> 'Tuple[TypeVar]':
 
     Returns
     ----------
-    Tuple[TypeVar]
+    Tuple[TypeVar, ...]
         Either:
 
         * If this `PEP 585`_-compliant generic defines a ``__parameters__``
@@ -272,7 +299,12 @@ def get_hint_pep585_generic_typevars(hint: object) -> 'Tuple[TypeVar]':
     hint_bases = get_hint_pep585_generic_bases_unerased(hint)
 
     # Set of all type variables parametrizing these pseudo-superclasses.
-    hint_typevars = set()
+    #
+    # Note the following inefficient iteration *CANNOT* be reduced to an
+    # efficient set comprehension, as each get_hint_pep_typevars() call returns
+    # a tuple of type variables rather than single type variable to be added to
+    # this set.
+    hint_typevars: Set[type] = set()
 
     # For each such pseudo-superclass, add all type variables parametrizing
     # this pseudo-superclass to this set.
