@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -12,6 +12,7 @@ This private submodule is *not* intended for importation by downstream callers.
 
 # ....................{ IMPORTS                           }....................
 import typing
+from beartype.cave import NoneType
 from beartype.roar import (
     BeartypeDecorHintPepException,
     BeartypeDecorHintPepSignException,
@@ -19,7 +20,7 @@ from beartype.roar import (
 from beartype._util.cache.utilcachecall import callable_cached
 from beartype._util.hint.data.pep.utilhintdatapep import (
     HINT_PEP_SIGNS_TYPE,
-    HINT_PEP_SIGNS_TYPE_ORIGIN,
+    HINT_PEP_SIGNS_TYPE_ORIGIN_STDLIB,
 )
 from beartype._util.hint.data.pep.proposal.utilhintdatapep484 import (
     HINT_PEP484_BASE_FORWARDREF)
@@ -35,12 +36,12 @@ from beartype._util.hint.pep.proposal.utilhintpep484 import (
 from beartype._util.hint.pep.proposal.utilhintpep585 import (
     get_hint_pep585_generic_bases_unerased,
     get_hint_pep585_generic_typevars,
-    is_hint_pep585,
+    is_hint_pep585_builtin,
     is_hint_pep585_generic,
 )
-from typing import Generic, NewType, TypeVar
+from typing import Any, Generic, NewType, Optional, Tuple, TypeVar
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ MAPPINGS                          }....................
@@ -91,7 +92,7 @@ else:
         # non-standard "type_var" instance variable whose value is either
         # "typing.AnyStr", "str", or "bytes". Since only the former is an
         # actual type variable, however, we further test that condition.
-        if isinstance(hint, typing._TypeAlias):
+        if isinstance(hint, typing._TypeAlias):  # type: ignore[attr-defined]
             # If this value is a type variable, return the empty tuple.
             if is_hint_pep_typevar(hint.type_var):
                 return ()
@@ -160,7 +161,7 @@ get_hint_pep_args.__doc__ = '''
 # If the active Python interpreter targets at least Python >= 3.9, implement
 # this function to perform cray-cray logic. *sigh*
 if IS_PYTHON_AT_LEAST_3_9:
-    def get_hint_pep_typevars(hint: object) -> 'Tuple[TypeVar]':
+    def get_hint_pep_typevars(hint: object) -> Tuple[type, ...]:
 
         # Value of the "__parameters__" dunder attribute on this object if this
         # object defines this attribute *OR* "None" otherwise.
@@ -185,7 +186,7 @@ if IS_PYTHON_AT_LEAST_3_9:
 # Else if the active Python interpreter targets Python 3.7 or 3.8, implement
 # this function to access the "__parameters__" dunder instance variable.
 elif IS_PYTHON_AT_LEAST_3_7:
-    def get_hint_pep_typevars(hint: object) -> 'Tuple[TypeVar]':
+    def get_hint_pep_typevars(hint: object) -> Tuple[type, ...]:
 
         # Value of the "__parameters__" dunder attribute on this object if this
         # object defines this attribute *OR* the empty tuple otherwise. Note:
@@ -203,7 +204,7 @@ elif IS_PYTHON_AT_LEAST_3_7:
 #
 # Gods... this is horrible. Thanks for nuthin', Python 3.6.
 else:
-    def get_hint_pep_typevars(hint: object) -> 'Tuple[TypeVar]':
+    def get_hint_pep_typevars(hint: object) -> Tuple[type, ...]:
 
         # Avoid circular import dependencies.
         from beartype._util.hint.pep.utilhintpeptest import is_hint_pep_typevar
@@ -214,7 +215,7 @@ else:
         # non-standard "type_var" instance variable whose value is either
         # "typing.AnyStr", "str", or "bytes". Since only the former is an
         # actual type variable, however, we further test that condition.
-        if isinstance(hint, typing._TypeAlias):
+        if isinstance(hint, typing._TypeAlias):  # type: ignore[attr-defined]
             # If this value is a type variable, return a new 1-tuple containing
             # only this type variable.
             if is_hint_pep_typevar(hint.type_var):
@@ -264,7 +265,7 @@ get_hint_pep_typevars.__doc__ = '''
 
     Returns
     ----------
-    Tuple[TypeVar]
+    Tuple[TypeVar, ...]
         Either:
 
         * If this object defines a ``__parameters__`` dunder attribute, the
@@ -286,7 +287,7 @@ get_hint_pep_typevars.__doc__ = '''
 
 # ....................{ GETTERS ~ sign                    }....................
 @callable_cached
-def get_hint_pep_sign(hint: object) -> dict:
+def get_hint_pep_sign(hint: Any) -> object:
     '''
     **Sign** (i.e., arbitrary object) uniquely identifying the passed
     PEP-compliant type hint if this hint is PEP-compliant *or* raise an
@@ -308,7 +309,7 @@ def get_hint_pep_sign(hint: object) -> dict:
       class like :class:`list` or :class:`tuple` *or* an abstract base class
       (ABC) declared by the :mod:`collections.abc` submodule like
       :class:`collections.abc.Iterable` or :class:`collections.abc.Sequence`),
-      :class:`beartype.cave.HintPep585Type`.
+      :class:`beartype.cave.HintGenericSubscriptedType`.
     * If this hint is a **generic** (i.e., subclasses of the
       :class:`typing.Generic` abstract base class (ABC)),
       :class:`typing.Generic`. Note this includes `PEP 544`-compliant
@@ -316,7 +317,7 @@ def get_hint_pep_sign(hint: object) -> dict:
       which implicitly subclass the :class:`typing.Generic` ABC as well.
     * If this hint is any other class declared by either the :mod:`typing`
       module (e.g., :class:`typing.TypeVar`) *or* the :mod:`beartype.cave`
-      submodule (e.g., :class:`beartype.cave.HintPep585Type`), that class.
+      submodule (e.g., :class:`beartype.cave.HintGenericSubscriptedType`), that class.
     * If this hint is a **forward reference** (i.e., string or instance of the
       concrete :class:`typing.ForwardRef` class), :class:`typing.ForwardRef`.
     * If this hint is a **type variable** (i.e., instance of the concrete
@@ -403,10 +404,25 @@ def get_hint_pep_sign(hint: object) -> dict:
     die_unless_hint_pep(hint)
     # Else, this hint is PEP-compliant.
 
+    # Note that these tests are intentionally ordered in descending specificity
+    # (i.e., ascending genericity) rather than descending likelihood of a
+    # match, which would return erroneous signs in common cases.
+    #
+    # If this hint is the PEP 484-compliant "None" singleton, return the type
+    # of that singleton.
+    if hint is None:
+        return NoneType
+    # Else, this hint is *NOT* the PEP 484-compliant "None" singleton.
+    #
     # If this hint is a PEP-compliant generic (i.e., class
     # superficially subclassing at least one non-class PEP-compliant object),
     # return the "typing.Generic" abstract base class (ABC) generically -- get
     # it? -- identifying PEP-compliant generics. Note that:
+    #
+    # * Generics *CANNOT* be detected by the general-purpose logic performed
+    #   below, as this ABC does *NOT* define a __repr__() dunder method
+    #   returning a string prefixed by the "typing." substring. Ergo, we
+    #   necessarily detect generics with an explicit test instead.
     # * *ALL* PEP 484-compliant generics and PEP 544-compliant protocols are
     #   guaranteed by the "typing" module to subclass this ABC regardless of
     #   whether those generics originally did so explicitly. How? By type
@@ -424,21 +440,17 @@ def get_hint_pep_sign(hint: object) -> dict:
     # Ergo, this ABC uniquely identifies many but *NOT* all generics. Although
     # non-ideal, the failure of PEP 585-compliant generics to subclass a common
     # superclass leaves us with little alternative.
-    #
-    # Note that generics *CANNOT* be detected by the general-purpose logic
-    # performed below, as this ABC does *NOT* define a __repr__() dunder method
-    # returning a string prefixed by the "typing." substring.
-    if is_hint_pep_generic(hint):
+    elif is_hint_pep_generic(hint):
         return Generic
     # Else, this hint is *NOT* a generic.
     #
     # If this hint is a PEP 585-compliant type hint, return the origin type
     # originating this hint (e.g., "list" for "list[str]").
     #
-    # Note that the get_hint_pep_type_origin() getter is intentionally *NOT*
+    # Note that the get_hint_pep_stdlib_type() getter is intentionally *NOT*
     # called here. Why? Because doing so would induce an infinite recursive
     # loop, since that function internally calls this function. *sigh*
-    elif is_hint_pep585(hint):
+    elif is_hint_pep585_builtin(hint):
         return hint.__origin__
     # Else, this hint is *NOT* a PEP 585-compliant type hint.
     #
@@ -528,7 +540,7 @@ def get_hint_pep_sign(hint: object) -> dict:
     # "AttributeError" exception if something goes horribly awry.
     #
     # Gods... this is horrible. Thanks for nuthin', Python 3.6.
-    elif IS_PYTHON_3_6 and isinstance(hint, typing._TypeAlias):
+    elif IS_PYTHON_3_6 and isinstance(hint, typing._TypeAlias):  # type: ignore[attr-defined]
         return getattr(typing, hint.name)
     # Else, this hint *MUST* be a standard PEP 484-compliant type hint defined
     # by the "typing" module.
@@ -597,14 +609,77 @@ def get_hint_pep_sign(hint: object) -> dict:
     # Return this "typing" attribute.
     return sign
 
-# ....................{ GETTERS ~ type                    }....................
-def get_hint_pep_type_origin(hint: object) -> type:
+# ....................{ GETTERS ~ type : generic          }....................
+@callable_cached
+def get_hint_pep_generic_type_or_none(hint: Any) -> Optional[type]:
     '''
-    **Origin type** (i.e., non-:mod:`typing` class such that *all* objects
-    satisfying the passed PEP-compliant type hint are instances of this class)
-    originating this hint if this hint originates from a non-:mod:`typing`
-    class *or* raise an exception otherwise (i.e., if this hint does *not*
-    originate from a non-:mod:`typing` class).
+    Either the passed **generic** (i.e., class superficially subclassing at
+    least one PEP-compliant type hint that is possibly *not* an actual class)
+    if **unsubscripted** (i.e., indexed by *no* arguments or type variables),
+    the unsubscripted generic underlying this generic if **subscripted** (i.e.,
+    indexed by one or more arguments and/or type variables), *or* ``None``
+    otherwise (i.e., if this hint is *not* a generic).
+
+    Specifically, this getter returns:
+
+    * If this hint both originates from an **origin type** (i.e.,
+      non-:mod:`typing` class such that *all* objects satisfying this hint are
+      instances of that class), that origin type.
+    * Else if this hint is already a class, this hint as is.
+    * Else, ``None``.
+
+    This getter is memoized for efficiency.
+
+    Caveats
+    ----------
+    **This getter returns false positives in edge cases.** That is, this getter
+    returns non-``None`` values for both generics and non-generics. Callers
+    *must* perform subsequent tests to distinguish these two cases.
+
+    Parameters
+    ----------
+    hint : object
+        Object to be inspected.
+
+    Returns
+    ----------
+    Optional[type]
+        Either:
+
+        * If this hint is a generic, the class originating this generic.
+        * Else, ``None``.
+
+    See Also
+    ----------
+    :func:`_get_hint_pep_origin_object_or_none`
+        Further details.
+    '''
+
+    # Arbitrary object originating this hint if any *OR* "None" otherwise.
+    hint_origin = _get_hint_pep_origin_object_or_none(hint)
+    # print(f'{repr(hint)} hint_origin: {repr(hint_origin)}')
+
+    # If this object is a class, this object is the origin type originating
+    # this hint. In this case, return this object.
+    if isinstance(hint_origin, type):
+        return hint_origin
+    # Else if this hint is already a class, this class is effectively already
+    # its origin type. In this case, return this class as is.
+    elif isinstance(hint, type):
+        return hint
+    # Else, this hint is *NOT* a class. In this case, this hint originates from
+    # *NO* origin type.
+    else:
+        return None
+
+# ....................{ GETTERS ~ type : stdlib           }....................
+def get_hint_pep_stdlib_type(hint: object) -> type:
+    '''
+    **Standard library origin type** (i.e., non-:mod:`typing` class such that
+    *all* objects satisfying the passed PEP-compliant type hint are instances
+    of this class) originating this hint if this hint originates from a
+    non-:mod:`typing` class *or* raise an exception otherwise (i.e., if this
+    hint does *not* originate from a standard library origin type).
 
     This getter is intentionally *not* memoized (e.g., by the
     :func:`callable_cached` decorator), as the implementation trivially reduces
@@ -617,22 +692,27 @@ def get_hint_pep_type_origin(hint: object) -> type:
 
     Returns
     ----------
-    tuple
-        Origin type originating this hint.
+    type
+        Standard library origin type originating this hint.
+
+    Raises
+    ----------
+    BeartypeDecorHintPepException
+        If this hint does *not* originate from a standard library origin type.
 
     See Also
     ----------
-    :func:`get_hint_pep_type_origin_or_none`
+    :func:`get_hint_pep_stdlib_type_or_none`
     '''
 
     # Origin type originating this object if any *OR* "None" otherwise.
-    hint_type_origin = get_hint_pep_type_origin_or_none(hint)
+    hint_type_origin = get_hint_pep_stdlib_type_or_none(hint)
 
     # If this type does *NOT* exist, raise an exception.
     if hint_type_origin is None:
         raise BeartypeDecorHintPepException(
             f'PEP type hint {repr(hint)} not originative '
-            f'(i.e., does not originate from a lower-level class).'
+            f'(i.e., does not originate from external class).'
         )
     # Else, this type exists.
 
@@ -640,82 +720,14 @@ def get_hint_pep_type_origin(hint: object) -> type:
     return hint_type_origin
 
 
-# If the active Python interpreter targets at least Python >= 3.7, implement
-# this function to access the standard "__origin__" dunder instance variable
-# whose value is the origin type originating this hint.
-if IS_PYTHON_AT_LEAST_3_7:
-    def get_hint_pep_type_origin_or_none(hint: object) -> 'Optional[type]':
-
-        # Sign uniquely identifying this hint.
-        hint_sign = get_hint_pep_sign(hint)
-
-        # If this sign originates from an origin type...
-        if hint_sign in HINT_PEP_SIGNS_TYPE_ORIGIN:
-            # Return either...
-            return (
-                # If this hint is PEP 585-compliant type hint, this sign. By
-                # definition, the sign uniquely identifying *EVERY* PEP
-                # 585-compliant type hint is the origin type originating that
-                # hint (e.g., "list" for "list[str]").
-                hint_sign
-                if is_hint_pep585(hint) else
-                # Else, this hint is *NOT* a PEP 585-compliant type hint. By
-                # elimination, this hint *MUST* be a PEP 484-compliant type
-                # hint. In this case, return the origin type originating this
-                # hint (e.g., "list" for "typing.List[str]").
-                hint.__origin__
-            )
-
-        # Else, this sign does *NOT* originate from an origin type. In this
-        # case, return "None".
-        return None
-
-#FIXME: Drop this like hot lead after dropping Python 3.6 support.
-# Else, the active Python interpreter targets Python 3.6. In this case...
-#
-# Gods... this is horrible. Thanks for nuthin', Python 3.6.
-else:
-    def get_hint_pep_type_origin_or_none(hint: object) -> 'Optional[type]':
-
-        # Sign uniquely identifying this hint.
-        hint_sign = get_hint_pep_sign(hint)
-
-        # If this sign originates from an origin type...
-        #
-        # Note this could be implemented substantially more efficiently -- but
-        # why even bother? Python 3.6 is on unofficial intubation and will be
-        # officially terminated shortly.
-        if hint_sign in HINT_PEP_SIGNS_TYPE_ORIGIN:
-            # If this hint is a poorly designed Python 3.6-specific "type
-            # alias", this hint is a subscription of either the "typing.Match"
-            # or "typing.Pattern" objects. In this case, this hint declares a
-            # non-standard "impl_type" instance variable whose value is either
-            # the "re.Match" or "re.Pattern" class. Return this class.
-            if isinstance(hint, typing._TypeAlias):
-                return hint.impl_type
-
-            # Else, this hint is a poorly designed Python 3.6-specific "generic
-            # meta." In this case, this hint declares a non-standard
-            # "__extra__" dunder instance variable whose value is the origin
-            # type originating this hint. The "__origin__" dunder instance
-            # variable *DOES* exist under Python 3.6 but is typically the
-            # identity class referring to the same "typing" singleton (e.g.,
-            # "typing.List" for "typing.List[int]") rather than the origin type
-            # (e.g., "list" for "typing.List[int]") and is thus completely
-            # useless for everything.
-            return hint_sign.__extra__
-
-        # Else, this sign does *NOT* originate from an origin type. In this
-        # case, return "None".
-        return None
-
-# Document this function regardless of implementation details above.
-get_hint_pep_type_origin_or_none.__doc__ = '''
-    **Origin type** (i.e., non-:mod:`typing` class such that *all* objects
-    satisfying the passed PEP-compliant type hint are instances of this class)
-    originating this hint if this hint originates from a non-:mod:`typing`
-    class *or* ``None`` otherwise (i.e., if this hint does *not* originate from
-    a non-:mod:`typing` class).
+def get_hint_pep_stdlib_type_or_none(hint: Any) -> Optional[type]:
+    '''
+    **Standard library origin type** (i.e., non-:mod:`typing` class defined by
+    Python's standard library such that *all* objects satisfying the passed
+    PEP-compliant type hint are instances of this class) originating this hint
+    if this hint originates from a non-:mod:`typing` class *or* ``None``
+    otherwise (i.e., if this hint does *not* originate from a standard library
+    origin type).
 
     This getter is intentionally *not* memoized (e.g., by the
     :func:`callable_cached` decorator), as the implementation trivially reduces
@@ -723,21 +735,9 @@ get_hint_pep_type_origin_or_none.__doc__ = '''
 
     Caveats
     ----------
-    **This function should always be called in lieu of attempting to directly
-    access the low-level** ``__origin__`` **dunder attribute.** This attribute
-    is defined non-orthogonally by various singleton objects in the
-    :mod:`typing` module, including:
-
-    * Objects failing to define this attribute (e.g., :attr:`typing.Any`,
-      :attr:`typing.NoReturn`).
-    * Objects defining this attribute to be their unsubscripted :mod:`typing`
-      object (e.g., :attr:`typing.Optional`, :attr:`typing.Union`).
-    * Objects defining this attribute to be their origin type.
-
-    Since the :mod:`typing` module neither guarantees the existence of this
-    attribute nor imposes a uniform semantic on this attribute when defined,
-    this attribute is *not* safely directly accessible. Thus this function,
-    which "fills in the gaps" by implementing this oversight.
+    **This high-level getter should always be called in lieu of the low-level**
+    :func:`_get_hint_pep_origin_object_or_none` **getter or attempting to
+    directly access the low-level** ``__origin__`` **dunder attribute.**
 
     Parameters
     ----------
@@ -746,40 +746,33 @@ get_hint_pep_type_origin_or_none.__doc__ = '''
 
     Returns
     ----------
-    tuple
+    Optional[type]
         Either:
 
-        * If this object originates from a non-:mod:`typing` class, that class.
+        * If this hint originates from a standard library origin type, that
+          type.
         * Else, ``None``.
 
-    Examples
+    See Also
     ----------
-        >>> import typing
-        >>> from beartype._util.hint.pep.utilhintpepget import (
-        ...     get_hint_pep_type_origin_or_none)
-        # This is sane.
-        >>> get_hint_pep_type_origin_or_none(typing.List[int])
-        list
-        >>> get_hint_pep_type_origin_or_none(typing.Union[int, str])
-        None
-        # This is reasonable.
-        >>> typing.List.__origin__
-        list
-        >>> typing.List[int].__origin__
-        list
-        # This is crazy.
-        >>> typing.Union.__origin__
-        AttributeError: '_SpecialForm' object has no attribute '__origin__'
-        # This is balls crazy.
-        >>> typing.Union[int].__origin__
-        AttributeError: type object 'int' has no attribute '__origin__'
-        # This is balls cray-cray -- the ultimate evolution of crazy.
-        >>> typing.Union[int, str].__origin__
-        typing.Union
+    :func:`_get_hint_pep_origin_object_or_none`
+        Further details.
     '''
 
-# ....................{ GETTERS ~ subtype : generic       }....................
-def get_hint_pep_generic_bases_unerased(hint: object) -> 'Tuple[object]':
+    # Sign uniquely identifying this hint.
+    hint_sign = get_hint_pep_sign(hint)
+
+    # Return either...
+    return (
+        # If this sign originates from an origin type, that type.
+        _get_hint_pep_origin_object_or_none(hint)
+        if hint_sign in HINT_PEP_SIGNS_TYPE_ORIGIN_STDLIB else
+        # Else, "None".
+        None
+    )
+
+# ....................{ GETTERS ~ kind : generic          }....................
+def get_hint_pep_generic_bases_unerased(hint: object) -> Tuple[object, ...]:
     '''
     Tuple of all **unerased pseudo-superclasses** (i.e., PEP-compliant objects
     originally listed as superclasses prior to their implicit type erasure
@@ -979,3 +972,136 @@ def get_hint_pep_generic_bases_unerased(hint: object) -> 'Tuple[object]':
 
     # Return this tuple of these pseudo-superclasses.
     return hint_pep_generic_bases_unerased
+
+# ....................{ PRIVATE ~ getters : type          }....................
+# If the active Python interpreter targets at least Python >= 3.7, implement
+# this function to access the standard "__origin__" dunder instance variable
+# whose value is the origin type originating this hint.
+if IS_PYTHON_AT_LEAST_3_7:
+    def _get_hint_pep_origin_object_or_none(hint: Any) -> Optional[Any]:
+
+        # Return this hint's origin object if any *OR* "None" otherwise.
+        return getattr(hint, '__origin__', None)
+
+#FIXME: Drop this like hot lead after dropping Python 3.6 support.
+# Else, the active Python interpreter targets Python 3.6. In this case...
+#
+# Gods... this is horrible. Thanks for nuthin', Python 3.6.
+else:
+    from beartype._util.utilobject import SENTINEL
+
+    def _get_hint_pep_origin_object_or_none(hint: Any) -> Optional[Any]:
+
+        # If this hint is a poorly designed Python 3.6-specific "type
+        # alias", this hint is a subscription of either the "typing.Match"
+        # or "typing.Pattern" objects. In this case, this hint declares a
+        # non-standard "impl_type" instance variable whose value is either
+        # the "re.Match" or "re.Pattern" class. Return that class.
+        if isinstance(hint, typing._TypeAlias):  # type: ignore[attr-defined]
+            return hint.impl_type
+
+        # This hint's origin object if any *OR* "None" otherwise.
+        #
+        # If this hint is a poorly designed Python 3.6-specific "generic meta,"
+        # this hint declares a non-standard "__extra__" dunder instance
+        # variable whose value is the origin object originating this hint. The
+        # "__origin__" dunder instance variable *DOES* exist under Python 3.6
+        # but is typically the identity class referring to the same "typing"
+        # singleton (e.g., "typing.List" for "typing.List[int]") rather than
+        # the origin type (e.g., "list" for "typing.List[int]") and is thus
+        # completely useless for everything.
+        #
+        # Note that we intentionally avoid defaulting "hint_extra" to "None"
+        # if that attribute is undefined. Under Python 3.6, user-defined PEP
+        # 484-compliant generics define:
+        # * "hint.__extra__" to be "None".
+        # * "hint.__origin__" to be the desired origin.
+        #
+        # Altogether, this means we *MUST* distinguish these two cases with a
+        # sentinel placeholder that is explicitly *NOT* "None".
+        #
+        # Welcome to Typing Hell. We hope you really enjoy pain, because... I
+        # mean, just look around.
+        hint_extra = getattr(hint, '__extra__', SENTINEL)
+        return (
+            hint_extra
+            if hint_extra is not None else
+            getattr(hint, '__origin__', None)
+        )
+
+# Document this function regardless of implementation details above.
+_get_hint_pep_origin_object_or_none.__doc__ = '''
+    **Unsafe origin object** (i.e., arbitrary object possibly related to the
+    passed PEP-compliant type hint but *not* necessarily a non-:mod:`typing`
+    class such that *all* objects satisfying this hint are instances of this
+    class) originating this hint if this hint originates from an object *or*
+    ``None`` otherwise (i.e., if this hint originates from *no* such object).
+
+    This getter is intentionally *not* memoized (e.g., by the
+    :func:`callable_cached` decorator), as the implementation trivially reduces
+    to an efficient one-liner.
+
+    Caveats
+    ----------
+    **The high-level** :func:`get_hint_pep_stdlib_type_or_none` function should
+    always be called in lieu of this low-level function.** Whereas the former
+    is guaranteed to return either a class or ``None``, this function enjoys no
+    such guarantees and instead returns what the caller can only safely assume
+    to be an arbitrary object.
+
+    If this function *must* be called, **this function should always be called
+    in lieu of attempting to directly access the low-level** ``__origin__``
+    **dunder attribute.** That attribute is defined non-orthogonally by various
+    singleton objects in the :mod:`typing` module, including:
+
+    * Objects failing to define this attribute (e.g., :attr:`typing.Any`,
+      :attr:`typing.NoReturn`).
+    * Objects defining this attribute to be their unsubscripted :mod:`typing`
+      object (e.g., :attr:`typing.Optional`, :attr:`typing.Union`).
+    * Objects defining this attribute to be their origin type.
+
+    Since the :mod:`typing` module neither guarantees the existence of this
+    attribute nor imposes a uniform semantic on this attribute when defined,
+    that attribute is *not* safely directly accessible. Thus this function,
+    which "fills in the gaps" by implementing this oversight.
+
+    Parameters
+    ----------
+    hint : object
+        Object to be inspected.
+
+    Returns
+    ----------
+    Optional[Any]
+        Either:
+
+        * If this hint originates from an arbitrary object, that object.
+        * Else, ``None``.
+
+    Examples
+    ----------
+        >>> import typing
+        >>> from beartype._util.hint.pep.utilhintpepget import (
+        ...     get_hint_pep_origin_type_unsafe_or_none)
+        # This is sane.
+        >>> get_hint_pep_origin_type_unsafe_or_none(typing.List)
+        list
+        >>> get_hint_pep_origin_type_unsafe_or_none(typing.List[int])
+        list
+        >>> get_hint_pep_origin_type_unsafe_or_none(typing.Union)
+        None
+        >>> get_hint_pep_origin_type_unsafe_or_none(typing.Union[int])
+        None
+        # This is insane.
+        >>> get_hint_pep_origin_type_unsafe_or_none(typing.Union[int, str])
+        Union
+        # This is crazy.
+        >>> typing.Union.__origin__
+        AttributeError: '_SpecialForm' object has no attribute '__origin__'
+        # This is balls crazy.
+        >>> typing.Union[int].__origin__
+        AttributeError: type object 'int' has no attribute '__origin__'
+        # This is balls cray-cray -- the ultimate evolution of crazy.
+        >>> typing.Union[int, str].__origin__
+        typing.Union
+    '''
