@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -10,37 +10,8 @@ validating arbitrary objects to be PEP-compliant type hints).
 This private submodule is *not* intended for importation by downstream callers.
 '''
 
-# ....................{ TODO                              }....................
-#FIXME: Refactor is_hint_pep_supported() and die_unless_hint_pep_supported() to
-#explicitly reject (with a human-readable exception in the latter case) PEP
-#544-compatible protocols *NOT* decorated by @typing.runtime_checkable, as such
-#PEP-compliant type hints are unusable at runtime.
-#
-#Alternately, we could always try something *REAL* sneaky and clever:
-#basically, rather than accept "typing" nonsense verbatim, we could instead:
-#* Detect PEP 544-compatible protocol type hints *NOT* decorated by
-#  @typing.runtime_checkable.
-#* Emit a non-fatal warning advising the end user to resolve this on their end.
-#* Meanwhile, beartype can simply:
-#  * Dynamically fabricate a new PEP 544-compatible protocol decorated by
-#    @typing.runtime_checkable using the body of the undecorated user-defined
-#    protocol as its base. Indeed, simply subclassing a new subclass decorated
-#    by @typing.runtime_checkable from the undecorated user-defined protocol as
-#    its base with a noop body of "pass" should suffice.
-#  * Replacing all instances of the undecorated user-defined protocol with that
-#    decorated beartype-defined protocol in annotations. Note this would
-#    strongly benefit from some form of memoization or caching. Since this edge
-#    case should be fairly rare, even a dictionary would probably be overkill.
-#    Just implementing something resembling the following memoized getter
-#    in the "utilhintpep544" submodule would probably suffice:
-#        @callable_cached
-#        def get_pep544_protocol_checkable_from_protocol_uncheckable(
-#            protocol_uncheckable: object) -> Protocol:
-#            ...
-#
-#Checkmate, "typing". Checkmate.
-
 # ....................{ IMPORTS                           }....................
+from beartype.cave import HintGenericSubscriptedType
 from beartype.roar import (
     BeartypeDecorHintPepException,
     BeartypeDecorHintPepDeprecatedWarning,
@@ -67,18 +38,18 @@ from beartype._util.hint.pep.proposal.utilhintpep484 import (
 from beartype._util.hint.pep.proposal.utilhintpep544 import (
     is_hint_pep544_ignorable_or_none)
 from beartype._util.hint.pep.proposal.utilhintpep585 import (
-    is_hint_pep585,
+    is_hint_pep585_builtin,
     is_hint_pep585_generic,
 )
 from beartype._util.hint.pep.proposal.utilhintpep593 import (
     is_hint_pep593_ignorable_or_none)
-from beartype._util.utilobject import get_object_class_unless_class
 from beartype._util.py.utilpymodule import get_object_module_name
 from beartype._util.py.utilpyversion import IS_PYTHON_AT_LEAST_3_7
+from beartype._util.utilobject import get_object_type_unless_type
 from typing import TypeVar
 from warnings import warn
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ CONSTANTS                         }....................
@@ -115,8 +86,8 @@ def die_if_hint_pep(
     hint: object,
 
     # Optional parameters.
-    hint_label: 'Optional[str]' = 'Type hint',
-    exception_cls: 'Optional[type]' = BeartypeDecorHintPepException,
+    hint_label: str = 'Type hint',
+    exception_cls: type = BeartypeDecorHintPepException,
 ) -> None:
     '''
     Raise an exception if the passed object is a **PEP-compliant type
@@ -152,7 +123,7 @@ def die_if_hint_pep(
 
         # Raise an exception of this class.
         raise exception_cls(
-            f'{hint_label} {repr(hint)} PEP-compliant '
+            f'{hint_label} {repr(hint)} PEP type hint '
             f'(e.g., rather than non-"typing" type).'
         )
 
@@ -190,8 +161,7 @@ def die_unless_hint_pep(
     if not is_hint_pep(hint):
         assert isinstance(hint_label, str), f'{repr(hint_label)} not string.'
         raise BeartypeDecorHintPepException(
-            f'{hint_label} {repr(hint)} not PEP-compliant '
-            f'(e.g., not "typing" type).')
+            f'{hint_label} {repr(hint)} not PEP type hint.')
 
 # ....................{ EXCEPTIONS ~ supported            }....................
 #FIXME: Refactor all or most calls to this and the
@@ -315,10 +285,15 @@ def die_if_hint_pep_sign_unsupported(
         assert isinstance(hint_label, str), f'{repr(hint_label)} not string.'
         raise BeartypeDecorHintPepUnsupportedException(
             f'{hint_label} PEP sign {repr(hint_sign)} '
-            f'currently unsupported by @beartype.'
-        )
+            f'currently unsupported by @beartype.')
 
 # ....................{ WARNINGS                          }....................
+#FIXME: Resurrect support for the passed "hint_label" parameter. We've
+#currently disabled this parameter as it's typically just a non-human-readable
+#placeholder substring *NOT* intended to be exposed to end users (e.g.,
+#"$%ROOT_PITH_LABEL/~"). For exceptions, we simply catch raised exceptions and
+#replace such substrings with human-readable equivalents. Can we perform a
+#similar replacement for warnings?
 def warn_if_hint_pep_sign_deprecated(
     # Mandatory parameters.
     hint: object,
@@ -326,7 +301,7 @@ def warn_if_hint_pep_sign_deprecated(
 
     # Optional parameters.
     hint_label: str = 'Annotated',
-) -> bool:
+) -> None:
     '''
     Emit a non-fatal warning only if the passed PEP-compliant sign uniquely
     identifying the passed PEP-compliant type hint is **deprecated** (e.g., due
@@ -355,10 +330,20 @@ def warn_if_hint_pep_sign_deprecated(
 
     # If this sign is deprecated...
     if hint_sign in HINT_PEP_SIGNS_DEPRECATED:
-        assert isinstance(hint_label, str), f'{repr(hint_label)} not string.'
+        #FIXME: Uncomment *AFTER* resolving the "FIXME:" above.
+        #FIXME: Unit test that this string contains *NO* non-human-readable
+        #placeholder substrings. Note that the existing
+        #"beartype_test.a00_unit.decor.code.test_codemain" submodule contains
+        #relevant logic currently disabled for reasons that hopefully no longer
+        #apply. *Urgh!*
+
+        # assert isinstance(hint_label, str), f'{repr(hint_label)} not string.'
+        #
+        # # Warning message to be emitted.
+        # warning_message = f'{hint_label} PEP type hint {repr(hint)} deprecated'
 
         # Warning message to be emitted.
-        warning_message = f'{hint_label} PEP type hint {repr(hint)} deprecated'
+        warning_message = f'PEP type hint {repr(hint)} deprecated'
 
         # If this sign uniquely identifies PEP 484-compliant type hints
         # originating from origin types (e.g., "typing.List[int]"), this sign
@@ -501,7 +486,7 @@ def is_hint_pep(hint: object) -> bool:
 
     # Either the passed object if this object is a class *OR* the class of this
     # object otherwise (i.e., if this object is *NOT* a class).
-    hint_type = get_object_class_unless_class(hint)
+    hint_type = get_object_type_unless_type(hint)
 
     # Return true only if either...
     #
@@ -509,11 +494,13 @@ def is_hint_pep(hint: object) -> bool:
     # of a match with the least common type hints tested last and the most
     # common type hints tested first.
     return (
+        # This hint is the PEP 484-compliant "None" singleton.
+        hint is None or
         # This hint's type is directly declared by the "typing" module and thus
         # PEP-compliant by definition *OR*...
-        is_hint_pep_class_typing(hint_type) or
+        is_hint_pep_type_typing(hint_type) or
         # This hint is a PEP 585-compliant type hint.
-        is_hint_pep585(hint) or
+        is_hint_pep585_builtin(hint) or
         # This hint is a PEP-compliant generic. Although a small subset of
         # generics are directly defined by the "typing" module (e.g.,
         # "typing.SupportsInt"), most generics are user-defined subclasses
@@ -533,6 +520,75 @@ def is_hint_pep(hint: object) -> bool:
         is_hint_pep484_newtype(hint)
     )
 
+
+#FIXME: Unit test us up.
+def is_hint_pep_uncached(hint: object) -> bool:
+    '''
+    ``True`` only if the passed object is a **PEP-compliant non-self-cached
+    type hint** (i.e., PEP-compliant type hint that does *not* externally cache
+    itself somewhere).
+
+    Since most PEP-compliant type hints are self-caching, this function
+    returns:
+
+    * ``True`` for only:
+
+      * `PEP 484`_-compliant subscripted generics under Python >= 3.9 (e.g.,
+        ``from typing import List; class MuhPep484List(List): pass;
+        MuhPep484List[int]``). See below for further commentary.
+      * `PEP 585`_-compliant type hints, including both:
+
+        * Builtin `PEP 585`_-compliant type hints (e.g., ``list[int]``).
+        * User-defined `PEP 585`_-compliant generics (e.g.,
+          ``class MuhPep585List(list): pass; MuhPep585List[int]``).
+
+    * ``False`` for *all* other PEP-compliant type hints.
+
+    Caveats
+    ----------
+    This function *cannot* be meaningfully memoized, since the passed type hint
+    is *not* guaranteed to be cached somewhere. Only functions passed cached
+    type hints can be meaningfully memoized. Since this high-level function
+    internally defers to unmemoized low-level functions that are ``O(n)`` in
+    ``n`` the size of the inheritance hierarchy of this hint, this function
+    should be called sparingly. See the :mod:`beartype._decor._cache.cachehint`
+    submodule for further details.
+
+    This tester intentionally returns a false negative for `PEP 484`_-compliant
+    generics subscripted by type variables under Python < 3.9. Although those
+    hints are technically non-self-cached, this tester falsely reports those
+    hints to be self-cached by returning ``False``. Why? Because correctly
+    detecting those hints as non-self-cached would require an unmemoized
+    ``O(n)`` search across the inheritance hierarchy of *all* passed objects
+    and thus all type hints annotating callables decorated by
+    :func:`beartype.beartype`. Since this failure only affects obsolete Python
+    versions *and* since the only harms induced by this failure are a slight
+    increase in space and time consumption for edge-case type hints unlikely to
+    actually be used in real-world code, this tradeoff is more than acceptable.
+    We're not the bad guy here. Right?
+
+    Parameters
+    ----------
+    hint : object
+        Object to be inspected.
+
+    Returns
+    ----------
+    bool
+        ``True`` only if this object is a PEP-compliant type hint.
+
+    .. _PEP 484:
+        https://www.python.org/dev/peps/pep-0484
+    .. _PEP 585:
+        https://www.python.org/dev/peps/pep-0585
+    '''
+
+    # Return true only if this hint is either:
+    # * A PEP 585-compliant type hint.
+    # * A PEP 484-compliant subscripted generic masquerading as a PEP
+    #   585-compliant type hint. *shrug*
+    return is_hint_pep585_builtin(hint)
+
 # ....................{ TESTERS ~ ignorable               }....................
 def is_hint_pep_ignorable(hint: object) -> bool:
     '''
@@ -568,7 +624,6 @@ def is_hint_pep_ignorable(hint: object) -> bool:
         superficially appearing to do so.
     '''
 
-
     # Avoid circular import dependencies.
     from beartype._util.hint.pep.utilhintpepget import get_hint_pep_sign
 
@@ -669,7 +724,7 @@ def is_hint_pep_supported(hint: object) -> bool:
     return is_hint_pep_sign_supported(hint_pep_sign)
 
 
-def is_hint_pep_sign_supported(hint) -> bool:
+def is_hint_pep_sign_supported(hint: object) -> bool:
     '''
     ``True`` only if the passed object is a **PEP-compliant supported
     unsubscripted typing attribute** (i.e., public attribute of the
@@ -701,6 +756,7 @@ def is_hint_pep_sign_supported(hint) -> bool:
     '''
     # from beartype._util.hint.data.pep.utilhintdatapep import (
     #     HINT_PEP_SIGNS_SUPPORTED_DEEP)
+    # print(f'HINT_PEP_SIGNS_SUPPORTED: {HINT_PEP_SIGNS_SUPPORTED}')
     # print(f'HINT_PEP_SIGNS_SUPPORTED_DEEP: {HINT_PEP_SIGNS_SUPPORTED_DEEP}')
 
     # Return true only if this hint is a supported unsubscripted "typing"
@@ -708,6 +764,16 @@ def is_hint_pep_sign_supported(hint) -> bool:
     return hint in HINT_PEP_SIGNS_SUPPORTED
 
 # ....................{ TESTERS ~ typing                  }....................
+#FIXME: This test returns false negatives for PEP 593-compliant
+#"typing.Annotated" parent type hints, whose "__module__" attribute is
+#inexplicably that of the first PEP-compliant child type hint subscripting that
+#parent. Since the existing test suffices in all other cases *AND* since we
+#consider this behaviour aberrant rather than desirable, we are inclined to
+#preserve the existing test. If we ever *DO* want to properly support PEP
+#593-compliant type hints here, the correct solution will be to generalize this
+#tester as follows:
+#* Memoize this tester with @callable_cached.
+#* Generalize the body of this tester to:
 def is_hint_pep_typing(hint: object) -> bool:
     '''
     ``True`` only if the passed object is defined by the :mod:`typing` module.
@@ -726,14 +792,38 @@ def is_hint_pep_typing(hint: object) -> bool:
     bool
         ``True`` only if this object is defined by the :mod:`typing` module.
     '''
+    # print(f'is_hint_pep_typing({repr(hint)}')
 
-    return repr(hint).startswith('typing.')
+    # Return true if either...
+    return (
+        # This is any PEP-compliant type hint defined by the "typing" module
+        # *OTHER* than a PEP 593-compliant type hint, in which case the
+        # fully-qualified name of the module defining this hint is "typing".
+        getattr(hint, '__module__', None) == 'typing' or
+        # This is a PEP 593-compliant type hint. For inexplicable (and
+        # presumably indefensible) reasons, PEP 593-compliant type hints badly
+        # masquerade as their first subscripted PEP-compliant type hint (e.g.,
+        # the "int" in "typing.Annotated[int, 63]"). Ergo, the value of the
+        # "__module__" attribute of this hint is that of its first subscripted
+        # PEP-compliant type hint rather than its own. Nonetheless, its
+        # machine-readable representation remains prefixed by "typing.",
+        # enabling an efficient test that also generalizes to all other
+        # outlier edge cases that are probably lurking about.
+        #
+        # I have no code and I must scream.
+        repr(hint).startswith('typing.')
+    )
 
 
 # If the active Python interpreter targets at least Python 3.7 and is thus
 # sane enough to support the normal implementation of this tester, do so.
 if IS_PYTHON_AT_LEAST_3_7:
-    def is_hint_pep_class_typing(hint: object) -> bool:
+    def is_hint_pep_type_typing(hint: object) -> bool:
+
+        # This hint if this hint is a class *OR* this hint's class otherwise.
+        hint_type = get_object_type_unless_type(hint)
+        # print(f'pep_type_typing({repr(hint)}): {get_object_module_name(hint_type)}')
+
         # Return true only if this type is defined by the "typing" module.
         #
         # Note that this implementation could probably be reduced to the
@@ -782,8 +872,7 @@ if IS_PYTHON_AT_LEAST_3_7:
         #   exceptions with reliable messages across *ALL* Python versions.
         #
         # In short, there is no general-purpose clever solution. *sigh*
-        return get_object_module_name(get_object_class_unless_class(hint)) == (
-            'typing')
+        return hint_type.__module__ == 'typing'
 # Else, the active Python interpreter targets exactly Python 3.6. In this case,
 # define this tester to circumvent Python 3.6-specific issues. Notably, the
 # implementation of the "typing" module under this major version harmfully
@@ -795,12 +884,16 @@ if IS_PYTHON_AT_LEAST_3_7:
 # the resolution is simply to explicitly test for and reject "collections.abc"
 # superclasses passed to this Python 3.6-specific tester implementation.
 else:
-    def is_hint_pep_class_typing(hint: object) -> bool:
+    def is_hint_pep_type_typing(hint: object) -> bool:
+
+        # This hint if this hint is a class *OR* this hint's class otherwise.
+        hint_type = get_object_type_unless_type(hint)
+        # print(f'pep_type_typing({repr(hint)}): {repr(hint_type)} {repr(get_object_module_name(hint_type))}')
+
         # Return true only if...
         return (
             # This type pretends to be defined by the "typing" module *AND*...
-            get_object_module_name(get_object_class_unless_class(hint)) == (
-                'typing') and
+            get_object_module_name(hint_type) == 'typing' and
             # This type is *NOT* actually a superclass defined by the
             # "collections.abc" submodule. Ideally, we would simply uncomment
             # the following test:
@@ -823,8 +916,8 @@ else:
             repr(hint) != "<class 'typing.Generator'>"
         )
 
-# Docstring for this function regardless of implementation details.
-is_hint_pep_class_typing.__doc__ = '''
+
+is_hint_pep_type_typing.__doc__ = '''
     ``True`` only if either the passed object is defined by the :mod:`typing`
     module if this object is a class *or* the class of this object is defined
     by the :mod:`typing` module otherwise (i.e., if this object is *not* a
@@ -850,19 +943,14 @@ is_hint_pep_class_typing.__doc__ = '''
           module.
     '''
 
-# ....................{ TESTERS ~ subtype : generic       }....................
-def is_hint_pep_generic(hint: object) -> bool:
+# ....................{ TESTERS ~ subscript               }....................
+#FIXME: Consider removal, as we don't actually call this anywhere.
+def is_hint_pep_subscripted(hint: object) -> bool:
     '''
-    ``True`` only if the passed object is a **generic** (i.e., class
-    superficially subclassing at least one non-class PEP-compliant object).
-
-    Specifically, this tester returns ``True`` only if this object is a class
-    that is either:
-
-    * A `PEP 585`_-compliant generic as tested by the lower-level
-      :func:`is_hint_pep585_generic` function.
-    * A `PEP 484`_-compliant generic as tested by the lower-level
-      :func:`is_hint_pep484_generic` function.
+    ``True`` only if the passed object is a PEP-compliant type hint
+    subscripted by one or more **arguments** (i.e., PEP-compliant child type
+    hints) and/or **type variables** (i.e., instances of the :class:`TypeVar`
+    class).
 
     This tester is intentionally *not* memoized (e.g., by the
     :func:`callable_cached` decorator), as the implementation trivially reduces
@@ -876,96 +964,40 @@ def is_hint_pep_generic(hint: object) -> bool:
     Returns
     ----------
     bool
-        ``True`` only if this object is a generic.
-
-    See Also
-    ----------
-    :func:`is_hint_pep_typevared`
-        Commentary on the relation between generics and parametrized hints.
-
-    .. _PEP 484:
-       https://www.python.org/dev/peps/pep-0484
-    .. _PEP 585:
-       https://www.python.org/dev/peps/pep-0585
+        ``True`` only if this object is a PEP-compliant type hint subscripted
+        by one or more arguments and/or type variables.
     '''
 
-    # Return true only if this hint is...
-    return (
-        # A class that is either...
-        isinstance(hint, type) and (
-            # A PEP 585-compliant generic *OR*...
-            is_hint_pep585_generic(hint) or
-            # A PEP 484-compliant generic.
-            is_hint_pep484_generic(hint)
-        )
+    # Avoid circular import dependencies.
+    from beartype._util.hint.pep.utilhintpepget import (
+        get_hint_pep_args,
+        get_hint_pep_typevars,
     )
 
-# ....................{ TESTERS ~ subtype : tuple         }....................
-def is_hint_pep_tuple_empty(hint: object) -> bool:
-    '''
-    ``True`` only if the passed object is a PEP-compliant **empty fixed-length
-    tuple hint** (i.e., PEP-compliant type hint constraining piths to be the
-    empty tuple).
-
-    This tester r
-
-    This tester is intentionally *not* memoized (e.g., by the
-    :func:`callable_cached` decorator), as this tester is only called under
-    fairly uncommon edge cases.
-
-    Motivation
-    ----------
-    Since type variables are not themselves types but rather placeholders
-    dynamically replaced with types by type checkers according to various
-    arcane heuristics, both type variables and types parametrized by type
-    variables warrant special-purpose handling.
-
-    Parameters
-    ----------
-    hint : object
-        Object to be inspected.
-
-    Returns
-    ----------
-    bool
-        ``True`` only if this object is a type variable.
-
-    .. _PEP 484:
-       https://www.python.org/dev/peps/pep-0484
-    '''
-
-    # Return true only if this hint resembles either the PEP 484- or
-    # 585-compliant fixed-length empty tuple type hint. Since there only exist
-    # two such hints *AND* comparison against these hints is mostly fast, this
-    # test is efficient in the general case.
-    #
-    # Note that this test may also be inefficiently performed by explicitly
-    # obtaining this hint's sign and then subjecting this hint to specific
-    # tests conditionally depending on which sign and thus PEP this hint
-    # complies with: e.g.,
-    #     # Return true only if this hint is either...
-    #     return true (
-    #         # A PEP 585-compliant "tuple"-based hint subscripted by no
-    #         # child hints *OR*...
-    #         (
-    #             hint_curr_sign is tuple and
-    #             hint_childs_len == 0
-    #         ) or
-    #         # A PEP 484-compliant "typing.Tuple"-based hint subscripted
-    #         # by exactly one child hint *AND* this child hint is the
-    #         # empty tuple,..
-    #         (
-    #             hint_curr_sign is Tuple and
-    #             hint_childs_len == 1 and
-    #             hint_childs[0] == ()
-    #         )
-    #     )
+    # Return true only if this hint is either...
     return (
-        hint == HINT_PEP585_TUPLE_EMPTY or
-        hint == HINT_PEP484_TUPLE_EMPTY
+        # A PEP-compliant subscripted generic under Python >= 3.9, including:
+        # * A PEP 484- or 585-compliant subscripted generic.
+        # * A PEP 585-compliant builtin type hint.
+        #
+        # Note this test is technically redundant, since all subscripted
+        # generics *MUST* necessarily be subscripted by one or more arguments
+        # and/or type variables, subsequently tested for. Nonetheless, this
+        # test efficiently reduces to a builtin call and is thus preferable.
+        isinstance(hint, HintGenericSubscriptedType) or
+        # Any other PEP-compliant type hint subscripted by one or more
+        # arguments and/or type variables. Note that this test is *NOT*
+        # reducible to merely:
+        #     bool(get_hint_pep_args(hint) or get_hint_pep_typevars(hint))
+        # Frankly, we have no idea why. We suspect we'd probably have to
+        # change the "or" operator in the above expression to the "+" operator,
+        # at which point the resulting operation is likely to be substantially
+        # slower than the simple series of tests performed here.
+        bool(get_hint_pep_args(hint)) or
+        bool(get_hint_pep_typevars(hint))
     )
 
-# ....................{ TESTERS ~ subtype : typevar       }....................
+# ....................{ TESTERS ~ subscript : typevar     }....................
 def is_hint_pep_typevar(hint: object) -> bool:
     '''
     ``True`` only if the passed object either is a PEP-compliant **type
@@ -1085,9 +1117,125 @@ def is_hint_pep_typevared(hint: object) -> bool:
     # Avoid circular import dependencies.
     from beartype._util.hint.pep.utilhintpepget import get_hint_pep_typevars
 
-    # Return true only if this is a "typing" type parametrized by one or more
-    # type variables, trivially detected by testing whether the tuple of all
-    # type variables parametrizing this "typing" type if this type is a generic
-    # alias (e.g., "typing._GenericAlias" subtype) *OR* the empty tuple
-    # otherwise is non-empty.
-    return len(get_hint_pep_typevars(hint)) > 0
+    # Return true only if this hint is parametrized by one or more type
+    # variables, trivially detected by testing whether the tuple of all type
+    # variables parametrizing this hint is non-empty.
+    return bool(get_hint_pep_typevars(hint))
+
+# ....................{ TESTERS ~ kind : generic          }....................
+@callable_cached
+def is_hint_pep_generic(hint: object) -> bool:
+    '''
+    ``True`` only if the passed object is a **generic** (i.e., class
+    superficially subclassing at least one PEP-compliant type hint that is
+    possibly *not* an actual class).
+
+    Specifically, this tester returns ``True`` only if this object is a class
+    that is either:
+
+    * A `PEP 585`_-compliant generic as tested by the lower-level
+      :func:`is_hint_pep585_generic` function.
+    * A `PEP 484`_-compliant generic as tested by the lower-level
+      :func:`is_hint_pep484_generic` function.
+
+    This tester is memoized for efficiency. Although the implementation
+    trivially reduces to a one-liner, constant factors associated with that
+    one-liner are non-negligible. Moreover, this tester is called frequently
+    enough to warrant its reduction to an efficient lookup.
+
+    Parameters
+    ----------
+    hint : object
+        Object to be inspected.
+
+    Returns
+    ----------
+    bool
+        ``True`` only if this object is a generic.
+
+    See Also
+    ----------
+    :func:`is_hint_pep_typevared`
+        Commentary on the relation between generics and parametrized hints.
+
+    .. _PEP 484:
+       https://www.python.org/dev/peps/pep-0484
+    .. _PEP 585:
+       https://www.python.org/dev/peps/pep-0585
+    '''
+
+    # Return true only if this hint is...
+    return (
+        # A PEP 484-compliant generic. Note this test trivially reduces to
+        # an O(1) operation and is thus tested first.
+        is_hint_pep484_generic(hint) or
+        # A PEP 585-compliant generic. Note this test is O(n) in n the
+        # number of pseudo-superclasses originally subclassed by this
+        # generic and is thus tested last.
+        is_hint_pep585_generic(hint)
+    )
+
+# ....................{ TESTERS ~ kind : tuple            }....................
+def is_hint_pep_tuple_empty(hint: object) -> bool:
+    '''
+    ``True`` only if the passed object is a PEP-compliant **empty fixed-length
+    tuple hint** (i.e., PEP-compliant type hint constraining piths to be the
+    empty tuple).
+
+    This tester r
+
+    This tester is intentionally *not* memoized (e.g., by the
+    :func:`callable_cached` decorator), as this tester is only called under
+    fairly uncommon edge cases.
+
+    Motivation
+    ----------
+    Since type variables are not themselves types but rather placeholders
+    dynamically replaced with types by type checkers according to various
+    arcane heuristics, both type variables and types parametrized by type
+    variables warrant special-purpose handling.
+
+    Parameters
+    ----------
+    hint : object
+        Object to be inspected.
+
+    Returns
+    ----------
+    bool
+        ``True`` only if this object is a type variable.
+
+    .. _PEP 484:
+       https://www.python.org/dev/peps/pep-0484
+    '''
+
+    # Return true only if this hint resembles either the PEP 484- or
+    # 585-compliant fixed-length empty tuple type hint. Since there only exist
+    # two such hints *AND* comparison against these hints is mostly fast, this
+    # test is efficient in the general case.
+    #
+    # Note that this test may also be inefficiently performed by explicitly
+    # obtaining this hint's sign and then subjecting this hint to specific
+    # tests conditionally depending on which sign and thus PEP this hint
+    # complies with: e.g.,
+    #     # Return true only if this hint is either...
+    #     return true (
+    #         # A PEP 585-compliant "tuple"-based hint subscripted by no
+    #         # child hints *OR*...
+    #         (
+    #             hint_curr_sign is tuple and
+    #             hint_childs_len == 0
+    #         ) or
+    #         # A PEP 484-compliant "typing.Tuple"-based hint subscripted
+    #         # by exactly one child hint *AND* this child hint is the
+    #         # empty tuple,..
+    #         (
+    #             hint_curr_sign is Tuple and
+    #             hint_childs_len == 1 and
+    #             hint_childs[0] == ()
+    #         )
+    #     )
+    return (
+        hint == HINT_PEP585_TUPLE_EMPTY or
+        hint == HINT_PEP484_TUPLE_EMPTY
+    )
