@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -91,7 +91,8 @@ def join_delimited(
         return ''
     # If one string is passed, return this string as is.
     elif num_strs == 1:
-        return strs[0]
+        # This is clearly a string, yet mypy thinks it's Any
+        return strs[0]  # type: ignore[no-any-return]
     # If two strings are passed, return these strings delimited appropriately.
     elif num_strs == 2:
         return f'{strs[0]}{delimiter_if_two}{strs[1]}'
