@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -117,6 +117,13 @@ from enum import (
     EnumMeta as _EnumMeta,
 )
 from io import IOBase as _IOBase
+from typing import (
+    Any as _Any,
+    Dict as _Dict,
+    Union as _Union,
+    Tuple as _Tuple,
+    Type as _Type,
+)
 from weakref import (
     ref as _ref,
     ProxyTypes as _ProxyTypes,
@@ -146,9 +153,6 @@ from types import (
     ModuleType as _ModuleType,
 )
 
-# See the "beartype.__init__" submodule for further commentary.
-__all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
-
 # ....................{ IMPORTS ~ conditional             }....................
 #FIXME: Preserve for when we inevitably require similar logic in the future.
 
@@ -175,11 +179,11 @@ class UnavailableType(object):
     third-party dependencies).
     '''
 
-    def __instancecheck__(self, obj):
+    def __instancecheck__(self, obj) -> None:
         raise _BeartypeCallCheckUnavailableTypeException(
             f'{self} not passable as the second parameter to isinstance().')
 
-    def __subclasscheck__(self, cls):
+    def __subclasscheck__(self, cls) -> None:
         raise _BeartypeCallCheckUnavailableTypeException(
             f'{self} not passable as the second parameter to issubclass().')
 
@@ -199,7 +203,7 @@ Type of all objects regardless of type.
 '''
 
 
-NoneType = type(None)
+NoneType: _Any = type(None)
 '''
 Type of the ``None`` singleton.
 
@@ -252,16 +256,23 @@ by :class:`functools.partial` is C-based or pure Python -- only that some
 callable of indeterminate origin is in fact wrapped.
 '''
 
+
+CallableCodeObjectType: _Any = type(_get_type_or_unavailable.__code__)
+'''
+Type of all **code objects** (i.e., C-based objects underlying all pure-Python
+callables to which those callables are compiled for efficiency).
+'''
+
 # ....................{ TYPES ~ call : function           }....................
 FunctionType = _FunctionType
 '''
-Type of all **pure-Python functions** (i.e., functions implemented in pure
-Python *not* associated with an owning class or instance of a class).
+Type of all **pure-Python functions** (i.e., functions implemented in Python
+*not* associated with an owning class or instance of a class).
 
 Caveats
 ----------
-**This type ambiguously matches many callable types not commonly associated
-with "named functions,"** including:
+**This type ambiguously matches many callables not commonly associated with
+standard functions,** including:
 
 * **Lambda functions.** Of course, distinguishing between conventional named
   functions and unnamed lambda functions would usually be seen as overly
@@ -272,6 +283,14 @@ with "named functions,"** including:
   :func:`staticmethod` decorator regardless of whether accessed on their
   declaring classes or associated instances).
 
+**This type matches no callables whatsoever under some non-CPython
+interpreters,** including:
+
+* PyPy, which unconditionally compiles *all* pure-Python functions into C-based
+  functions. Ergo, under PyPy, *all* functions are guaranteed to be of the type
+  :class:`FunctionOrMethodCType` regardless of whether those functions were
+  initially defined in Python or C.
+
 See Also
 ----------
 :class:`MethodBoundInstanceOrClassType`
@@ -314,7 +333,7 @@ the ``PyInstanceMethod_Type`` C type explicitly admits that:
 # standard "types.MethodWrapperType" object, this is of no benefit to older
 # versions of Python. Ergo, the type of an arbitrary method wrapper guaranteed
 # to *ALWAYS* exist is obtained instead.
-MethodBoundInstanceDunderCType = type(''.__add__)
+MethodBoundInstanceDunderCType: _Any = type(''.__add__)
 '''
 Type of all **C-based bound method wrappers** (i.e., callable objects
 implemented in low-level C, associated with special methods of builtin types
@@ -331,7 +350,7 @@ See Also
 # standard "types.ClassMethodDescriptorType" object, this is of no benefit to
 # older versions of Python. Ergo, the type of an arbitrary method descriptor
 # guaranteed to *ALWAYS* exist is obtained instead.
-MethodUnboundClassCType = type(dict.__dict__['fromkeys'])
+MethodUnboundClassCType: _Any = type(dict.__dict__['fromkeys'])
 '''
 Type of all **C-based unbound class method descriptors** (i.e., callable
 objects implemented in low-level C, associated with class methods of
@@ -347,7 +366,7 @@ explicitly passing the intended ``cls`` objects as their first parameters).
 # standard "types.WrapperDescriptorType" object, this is of no benefit to older
 # versions of Python. Ergo, the type of an arbitrary method descriptor
 # guaranteed to *ALWAYS* exist is obtained instead.
-MethodUnboundInstanceDunderCType = type(str.__add__)
+MethodUnboundInstanceDunderCType: _Any = type(str.__add__)
 '''
 Type of all **C-based unbound dunder method wrapper descriptors** (i.e.,
 callable objects implemented in low-level C, associated with dunder methods of
@@ -369,7 +388,7 @@ See Also
 # standard "types.MethodDescriptorType" object, this is of no benefit to older
 # versions of Python. Ergo, the type of an arbitrary method descriptor
 # guaranteed to *ALWAYS* exist is obtained instead.
-MethodUnboundInstanceNondunderCType = type(str.upper)
+MethodUnboundInstanceNondunderCType: _Any = type(str.upper)
 '''
 Type of all **C-based unbound non-dunder method descriptors** (i.e., callable
 objects implemented in low-level C, associated with non-dunder methods of
@@ -897,19 +916,45 @@ that enumeration's type and should be directly referenced as such: e.g.,
 '''
 
 # ....................{ TYPES ~ hint                      }....................
-HintPep585Type = UnavailableType
+HintGenericSubscriptedType: _Any = UnavailableType
 '''
-C-based type of all `PEP 585`_-compliant **type hint** (i.e., C-based type hint
-instantiated by subscripting either a concrete builtin container class like
-:class:`list` or :class:`tuple` *or* an abstract base class (ABC) declared by
-the :mod:`collections.abc` submodule like :class:`collections.abc.Iterable` or
-:class:`collections.abc.Sequence`) if the active Python interpreter targets at
-least Python 3.9.0 *or* :class:`UnavailableType` otherwise.
+C-based type of all subscripted generics if the active Python interpreter
+targets Python >= 3.9 *or* :class:`UnavailableType` otherwise.
 
-Note that *all* `PEP 585`_-compliant type hints are classes. Ergo, this C-based
-type is the class of classes and thus effectively a metaclass. It's probably
-best not to think too hard about that.
+Subscripted generics include:
 
+* `PEP 585`_-compliant **builtin type hints** (i.e., C-based type hints
+  instantiated by subscripting either a concrete builtin container class like
+  :class:`list` or :class:`tuple` *or* an abstract base class (ABC) declared by
+  the :mod:`collections.abc` submodule like :class:`collections.abc.Iterable`
+  or :class:`collections.abc.Sequence`). Since *all* `PEP 585`_-compliant
+  builtin type hints are classes, this C-based type is the class of those
+  classes and thus effectively itself a metaclass. It's probably best not to
+  think about that.
+* `PEP 484`_-compliant **subscripted generics** (i.e., user-defined classes
+  subclassing one or more `PEP 484`_-compliant type hints subsequently
+  subscripted by one or more PEP-compliant type hints).
+* `PEP 585`_-compliant **subscripted generics** (i.e., user-defined classes
+  subclassing one or more `PEP 585`_-compliant type hints subsequently
+  subscripted by one or more PEP-compliant type hints).
+
+Caveats
+----------
+**This low-level type ambiguously matches semantically unrelated PEP-compliant
+type hints,** rendering this type all but useless for most practical purposes.
+To distinguish between the various semantic types of hints ambiguously matched
+by this type, higher-level PEP-specific functions *must* be called instead.
+These include:
+
+* :func:`beartype._util.hint.pep.proposal.utilhintpep484.is_hint_pep484_generic`.
+  detecting `PEP 484`_-compliant generic type hints.
+* :func:`beartype._util.hint.pep.proposal.utilhintpep585.is_hint_pep585_builtin`.
+  detecting `PEP 585`_-compliant builtin type hints.
+* :func:`beartype._util.hint.pep.proposal.utilhintpep585.is_hint_pep585_generic`.
+  detecting `PEP 585`_-compliant generic type hints.
+
+.. _PEP 484:
+    https://www.python.org/dev/peps/pep-0484
 .. _PEP 585:
     https://www.python.org/dev/peps/pep-0585
 '''
@@ -917,7 +962,13 @@ best not to think too hard about that.
 # If the active Python interpreter targets at least Python >= 3.9 and thus
 # supports PEP 585, correctly declare this type.
 if _IS_PYTHON_AT_LEAST_3_9:
-    HintPep585Type = type(list[str])
+    HintGenericSubscriptedType = type(list[str])  # type: ignore[misc]
+
+#FIXME: Deprecate this. We previously published this ambiguously named type.
+#Perhaps use a module __getattr__() under whatever Python versions support it?
+#Note this requires Python >= 3.7, but that Python 3.6 will simply ignore that
+#dunder method, which is obviously acceptable, because Python 3.6 is dead.
+HintPep585Type = HintGenericSubscriptedType
 
 # ....................{ TYPES ~ scalar                    }....................
 BoolType = _BoolType
@@ -1135,7 +1186,7 @@ parsers parsing either top-level commands *or* subcommands of those commands.
 # nature of Python's regular expression support, this type is *NOT* publicly
 # exposed. While the private "re._pattern_type" attribute does technically
 # provide this type, it does so in a private and hence non-portable manner.
-RegexCompiledType = type(_re.compile(r''))
+RegexCompiledType: _Any = type(_re.compile(r''))
 '''
 Type of all **compiled regular expressions** (i.e., objects created and
 returned by the stdlib :func:`re.compile` function).
@@ -1143,14 +1194,14 @@ returned by the stdlib :func:`re.compile` function).
 
 
 #FIXME: Once Python 3.6 support is dropped, this can be safely reduced to:
-#     RegexCompiledType = re.Match
+#     RegexMatchType = re.Match
 
 # Yes, this type is required for type validation at module scope elsewhere.
 # Yes, this is the most time-efficient means of obtaining this type. No, this
 # type is *NOT* directly importable. Although this type's classname is
 # published to be "_sre.SRE_Match", the "_sre" C extension provides no such
 # type for pure-Python importation. So it goes.
-RegexMatchType = type(_re.match(r'', ''))
+RegexMatchType: _Any = type(_re.match(r'', ''))
 '''
 Type of all **regular expression match objects** (i.e., objects returned by the
 :func:`re.match` function).
@@ -1163,7 +1214,7 @@ Type of all **regular expression match objects** (i.e., objects returned by the
 
 # ....................{ TYPES ~ lib : numpy               }....................
 # Conditionally redefined by the "TUPLES ~ init" subsection below.
-NumpyArrayType = UnavailableType
+NumpyArrayType: type = UnavailableType
 '''
 Type of all **NumPy arrays** (i.e., instances of the concrete
 :class:`numpy.ndarray` class implemented in low-level C and Fortran) if
@@ -1173,7 +1224,7 @@ Type of all **NumPy arrays** (i.e., instances of the concrete
 
 
 # Conditionally redefined by the "TUPLES ~ init" subsection below.
-NumpyScalarType = UnavailableType
+NumpyScalarType: type = UnavailableType
 '''
 Type of all **NumPy scalars** (i.e., instances of the abstract
 :class:`numpy.generic` base class implemented in low-level C and Fortran) if
@@ -1192,7 +1243,6 @@ class _UnavailableTypesTuple(tuple):
     under the active Python interpreter, typically due to insufficient Python
     version or non-installed third-party dependencies).
     '''
-
     pass
 
 
@@ -1215,7 +1265,7 @@ Specifically, for any callable parameter or return type annotated with:
 '''
 
 # ....................{ TUPLES ~ core                     }....................
-NoneTypeOr = _NoneTypeOrType()
+NoneTypeOr: _Any = _NoneTypeOrType()
 '''
 **:class:``NoneType`` tuple factory** (i.e., dictionary mapping from arbitrary
 types or tuples of types to the same types or tuples of types concatenated with
@@ -1380,8 +1430,21 @@ ambiguous type of all C-based bound methods and non-method functions.
 CallableTypes = tuple(set(FunctionTypes) | set(MethodTypes))
 '''
 Tuple of all **callable types** (i.e., types whose instances are callable
-objects, including both built-in and user-defined functions, lambdas, methods,
-and method descriptors).
+objects implemented in either low-level C or high-level Python, including both
+built-in and user-defined functions, lambdas, methods, and method descriptors).
+'''
+
+
+CallableCTypes = (
+    FunctionOrMethodCType,
+    MethodBoundInstanceDunderCType,
+    MethodUnboundInstanceDunderCType,
+    MethodUnboundInstanceNondunderCType,
+    MethodUnboundClassCType,
+)
+'''
+Tuple of all **C-based callable types** (i.e., types whose instances are
+callable objects implemented in low-level C rather than high-level Python).
 '''
 
 
@@ -1448,7 +1511,7 @@ Perl) typically implicitly convert:
 
 # ....................{ TUPLES ~ version                  }....................
 # Conditionally expanded by the "TUPLES ~ init" subsection below.
-VersionComparableTypes = (tuple,)
+VersionComparableTypes: _Tuple[type, ...] = (tuple,)
 '''
 Tuple of all **comparable version types** (i.e., types suitable for use both as
 parameters to callables accepting arbitrary version specifiers *and* as
@@ -1477,7 +1540,7 @@ ironically, those types supported both under older but *not* newer versions of
 
 # ....................{ TUPLES ~ lib : numpy              }....................
 # Conditionally expanded by the "TUPLES ~ init" subsection below.
-SequenceOrNumpyArrayTypes = (SequenceType,)
+SequenceOrNumpyArrayTypes: _Tuple[type, ...] = (SequenceType,)
 '''
 Tuple of all **mutable** and **immutable sequence types** (i.e., both concrete
 and structural subclasses of the abstract :class:`collections.abc.Sequence`
@@ -1499,7 +1562,7 @@ See Also
 
 
 # Conditionally expanded by the "TUPLES ~ init" subsection below.
-SequenceMutableOrNumpyArrayTypes = (SequenceMutableType,)
+SequenceMutableOrNumpyArrayTypes: _Tuple[type, ...] = (SequenceMutableType,)
 '''
 Tuple of all **mutable sequence types** (i.e., both concrete and structural
 subclasses of the abstract :class:`collections.abc.Sequence` base class;
@@ -1521,7 +1584,7 @@ See Also
 
 # ....................{ TUPLES ~ lib : setuptools         }....................
 # Conditionally redefined by the "TUPLES ~ init" subsection below.
-SetuptoolsVersionTypes = UnavailableTypes
+SetuptoolsVersionTypes: _Tuple[type, ...] = UnavailableTypes
 '''
 Tuple of all **:mod:`setuptools`-specific version types** (i.e., types
 instantiated and returned by both the third-party
@@ -1561,7 +1624,7 @@ This tuple matches these types if :mod:`pkg_resources` is importable:
 
 # If NumPy is importable...
 try:
-    import numpy as _numpy
+    import numpy as _numpy  # type: ignore
 
     # Define NumPy-specific types.
     NumpyArrayType  = _numpy.ndarray
@@ -1577,12 +1640,12 @@ except:
 
 # If setuptools is importable, conditionally define setuptools-specific types.
 try:
-    import pkg_resources as _pkg_resources
+    import pkg_resources as _pkg_resources  # type: ignore
 
     # Define setuptools-specific types.
     SetuptoolsVersionTypes = (
-        _pkg_resources.packaging.version.Version,
-        _pkg_resources.packaging.version.LegacyVersion,
+        _pkg_resources.packaging.version.Version,        # type: ignore[attr-defined]
+        _pkg_resources.packaging.version.LegacyVersion,  # type: ignore[attr-defined]
     )
     VersionComparableTypes += SetuptoolsVersionTypes
 # Else, setuptools is unimportable. While this should typically *NEVER* be the
@@ -1668,3 +1731,160 @@ This includes:
   specify versions as instance variables convertible into both of the prior
   formats (e.g., ``SetuptoolsVersionTypes[0]('2.4.14.2.1.356.23')``).
 '''
+
+# ....................{ DUNDERS                           }....................
+# Intentionally defined last, as nobody wants to stumble into a full-bore rant
+# first thing in the morning.
+__all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
+'''
+Special list global referencing a single attribute guaranteed *not* to exist.
+
+The definition of this global effectively prohibits star imports from this
+submodule into downstream modules by raising an :class:`AttributeError`
+exception on the first attempt to do so: e.g.,
+
+.. code-block:: shell-session
+
+   >>> from beartype import *
+   AttributeError: module 'beartype' has no attribute 'STAR_IMPORTS_CONSIDERED_HARMFUL'
+
+All package submodules intentionally define similar ``__all__`` list globals.
+Why? Because ``__all__`` is antithetical to sane API design and facilitates
+antipatterns across the Python ecosystem, including well-known harms associated
+with star imports and lesser-known harms associated with the artificial notion
+of an ``__all__``-driven "virtual public API:" to wit,
+
+* **Competing standards.** Thanks to ``__all__``, Python now provides two
+  conflicting conceptions of what constitutes the public API for a package or
+  module:
+
+  * The **conventional public API.** By convention, all module- and
+    class-scoped attributes *not* prefixed by ``_`` are public and thus
+    comprise the public API. Indeed, the standard interpretation of star
+    imports for packages and modules defining *no* ``__all__`` list globals is
+    exactly this.
+  * The **virtual public API.** By mandate, all module-scoped attributes
+    explicitly listed by the ``__all__`` global are public and thus *also*
+    comprise the public API. Consider the worst case of a module artificially
+    constructing this global to list all private module-scoped attributes
+    prefixed by ``_``; then the intersection of the conventional and virtual
+    public APIs for that module would be the empty list and these two competing
+    standards would be the list negations of one another. Ergo, the virtual
+    public API has no meaningful relation to the conventional public API or any
+    public attributes actually defined by any package or module.
+
+  These conflicting notions are evidenced no more strongly than throughout the
+  Python stdlib itself. Some stdlib modules ignore all notions of a public or
+  private API altogether (e.g., the :mod:`inspect` module, which
+  unconditionally introspects all attributes of various types regardless of
+  nomenclature or listing in ``__all__``); others respect only the conventional
+  public API (e.g., the :mod:`xmlrpc` package, whose server implementation
+  ignores ``_``-prefixed methods); still others respect only the virtual public
+  API (e.g., the :mod:`pickletools` module, which conditionally introspects the
+  :mod:`pickle` module via its ``__all__`` list global); still others respect
+  either depending on context with bizarre exceptions (e.g., the :mod:`pydoc`
+  module, which ignores attributes excluded from ``__all__`` for packages and
+  modules defining ``__all__`` and otherwise ignores ``_``-prefixed attributes
+  excluding :class:`collections.namedtuple` instances, which are considered
+  public because... reasons).
+
+  Which of these conflicted interpretations is correct? None and all of them,
+  since there is no correct interpretation. This is bad. This is even badder
+  for packages contractually adhering to `semver (i.e., semantic versioning)
+  <semver_>`__, despite there existing no uniform agreement in the Python
+  community as to what constitutes a "public Python API."
+* **Turing completeness.** Technically, both the conventional and virtual
+  public APIs are defined only dynamically at runtime by the current
+  Turing-complete Python interpreter. Pragmatically, the conventional public
+  API is usually declared statically; those conventional public APIs that do
+  conditionally declare public attributes (e.g., to circumvent platform
+  portability concerns) often go to great and agonizing pains to declare a
+  uniform API with stubs raising exceptions on undefined edge cases. Deciding
+  the conventional public API for a package or module is thus usually trivial.
+  However, deciding the virtual public API for the same package or module is
+  often non-trivial or even infeasible. While many packages and modules
+  statically define ``__all__`` to be a simple context-independent list, others
+  dynamically append and extend ``__all__`` with context-dependent list
+  operations mystically depending on heterogeneous context *not* decidable at
+  authoring time -- including obscure incantations such as the active platform,
+  flags and options enabled at compilation time for the active Python
+  interpreter and C extensions, the conjunction of celestial bodies in
+  accordance with astrological scripture, and abject horrors like dynamically
+  extending the ``__all__`` exported from one submodule with the ``__all__``
+  exported from another not under the control of the author of the first.
+  (``__all__`` gonna ``__all__``, bro.)
+* **Extrinsic cognitive load.** To decide what constitutes the "public API" for
+  any given package or module, rational decision-making humans supposedly
+  submit to a sadistic heuristic resembling the following:
+
+  * Does the package in question define a non-empty ``__init__`` submodule
+    defining a non-empty ``__all__`` list global? If so, the attributes listed
+    by this global comprise that package's public API. Since ``__all__`` is
+    defined only at runtime by a Turing-complete interpreter, however,
+    deciding these attributes is itself a Turing-complete problem.
+  * Else, all non-underscored attributes comprise that package's public API.
+    Since these attributes are also defined only at runtime by a
+    Turing-complete interpreter, deciding these attributes is again a
+    Turing-complete problem -- albeit typically less so. (See above.)
+
+* **Redundancy.** ``__all__`` violates the DRY (Don't Repeat Yourself)
+  principle, thus inviting accidental desynchronization and omissions between
+  the conventional and virtual public APIs for a package or module. This leads
+  directly to...
+* **Fragility.** By definition, accidentally excluding a public attribute from
+  the conventional public API is infeasible; either an attribute is public by
+  convention or it isn't. Conversely, accidentally omitting a public attribute
+  from the virtual public API is a trivial and all-too-common mishap. Numerous
+  stdlib packages and modules do so. This includes the pivotal :mod:`socket`
+  module, whose implementation in the Python 3.6.x series accidentally excludes
+  the public :func:`socket.socketpair` function from ``__all__`` if and only if
+  the private :mod:`_socket` C extension also defines the same function -- a
+  condition with no reasonable justification. Or *is* there? Dare mo shiranai.
+* **Inconsistency.** Various modules and packages that declare ``__all__``
+  randomly exclude public attributes for spurious and frankly indefensible
+  reasons. This includes the stdlib :mod:`typing` module, whose preamble reads:
+
+      The pseudo-submodules 're' and 'io' are part of the public
+      namespace, but excluded from __all__ because they might stomp on
+      legitimate imports of those modules.
+
+  This is the worst of all possible worlds. A package or module either:
+
+  * Leave ``__all__`` undefined (as most packages and modules do).
+  * Prohibit ``__all__`` (as :mod:`beartype` does).
+  * Define ``__all__`` in a self-consistent manner conforming to
+    well-established semantics, conventions, and expectations.
+
+* **Nonconsensus.** No consensus exists amongst either stdlib developers or the
+  Python community as a whole as to the interpretation of ``__all__``.
+  Third-party authors usually ignore ``__all__`` with respect to its role in
+  declaring a virtual public API, instead treating ``__all__`` as a means of
+  restricting star imports to some well-defined subset of public attributes.
+  This includes SciPy, whose :attr:`scipy.__init__.__all__` list global
+  excludes most public subpackages of interest (e.g., :mod:`scipy.linalg`, a
+  subpackage of linear algebra routines) while paradoxically including some
+  public subpackages of little to no interest (e.g., :attr:`scipy.test`, a unit
+  test scaffold commonly run only by developers and maintainers).
+* **Infeasibility.** We have established that no two packages or modules
+  (including both stdlib and third-party) agree as to the usage of ``__all__``.
+  Respecting the virtual public API would require authors to ignore public
+  attributes omitted from ``__all__``, including those omitted by either
+  accident or due to conflicting interpretations of ``__all__``. Since this is
+  pragmatically infeasible *and* since upstream packages cannot reasonably
+  prohibit downstream packages from importing public attributes either
+  accidentally or intentionally excluded from ``__all__``, most authors
+  justifiably ignore ``__all__``. (*So should you.*)
+* **Insufficiency.** The ``__all__`` list global only applies to module-scoped
+  attributes; there exists no comparable special attribute for classes with
+  which to define a comparable "virtual public class API." Whereas the
+  conventional public API uniformly applies to *all* attributes regardless of
+  scoping, the virtual public API only applies to module-scoped attributes -- a
+  narrower and less broadly applicable use case.
+* **Unnecessity.** The conventional public API already exists. The virtual
+  public API offers no tangible benefits over the conventional public API while0
+  offering all the above harms. Under any rational assessment, the virtual
+  public API can only be "considered harmful."
+
+.. _semver:
+    https://semver.org
+'''
