@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # --------------------( LICENSE                           )--------------------
-# Copyright (c) 2014-2020 Cecil Curry.
+# Copyright (c) 2014-2021 Beartype authors.
 # See "LICENSE" for further details.
 
 '''
@@ -29,6 +29,7 @@ of Life (EoL) (e.g., Python 3.5) are explicitly unsupported.
 # installation. This includes all standard Python and package modules but
 # *NOT* third-party dependencies, which if currently uninstalled will only be
 # installed at some later time in the installation.
+#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 # WARNING: To avoid polluting the public module namespace, external attributes
 # should be locally imported at module scope *ONLY* under alternate private
 # names (e.g., "from argparse import ArgumentParser as _ArgumentParser" rather
@@ -36,8 +37,10 @@ of Life (EoL) (e.g., Python 3.5) are explicitly unsupported.
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
 import sys as _sys
+from beartype._util.py.utilpyinterpreter import IS_PYPY as _IS_PYPY
+from typing import Tuple as _Tuple
 
-# See the "beartype.__init__" submodule for further commentary.
+# See the "beartype.cave" submodule for further commentary.
 __all__ = ['STAR_IMPORTS_CONSIDERED_HARMFUL']
 
 # ....................{ METADATA                          }....................
@@ -91,14 +94,12 @@ Maximum minor stable version of this major version of Python currently released
 '''
 
 
-def _convert_version_str_to_tuple(version_str: str) -> tuple:
+def _convert_version_str_to_tuple(version_str: str) -> _Tuple[int, ...]:
     '''
     Convert the passed human-readable ``.``-delimited version string into a
     machine-readable version tuple of corresponding integers.
     '''
-    assert isinstance(version_str, str), (
-        '"{}" not a version string.'.format(version_str))
-
+    assert isinstance(version_str, str), f'"{version_str}" not version string.'
     return tuple(int(version_part) for version_part in version_str.split('.'))
 
 
@@ -157,10 +158,10 @@ if _sys.version_info[:3] < PYTHON_VERSION_MIN_PARTS:
 
     # Die ignominiously.
     raise RuntimeError(
-        '{} requires at least Python {}, but '
-        'the active interpreter only targets Python {}. '
-        'We feel unbearable sadness for you.'.format(
-            NAME, PYTHON_VERSION_MIN, _PYTHON_VERSION))
+        f'{NAME} requires at least Python {PYTHON_VERSION_MIN}, but '
+        f'the active interpreter only targets Python {_PYTHON_VERSION}. '
+        f'We feel unbearable sadness for you.'
+    )
 
 # ....................{ METADATA ~ version                }....................
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
@@ -187,7 +188,7 @@ if _sys.version_info[:3] < PYTHON_VERSION_MIN_PARTS:
 # For further details, see http://semver.org.
 #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
-VERSION = '0.5.1'
+VERSION = '0.6.0'
 '''
 Human-readable package version as a ``.``-delimited string.
 '''
@@ -207,6 +208,13 @@ By PyPI design, this string must *not* span multiple lines or paragraphs.
 '''
 
 # ....................{ METADATA ~ authors                }....................
+AUTHOR_EMAIL = 'leycec@gmail.com'
+'''
+Email address of the principal corresponding author (i.e., the principal author
+responding to public correspondence).
+'''
+
+
 AUTHORS = 'Cecil Curry, et al.'
 '''
 Human-readable list of all principal authors of this package as a
@@ -214,14 +222,18 @@ comma-delimited string.
 
 For brevity, this string *only* lists authors explicitly assigned copyrights.
 For the list of all contributors regardless of copyright assignment or
-attribution, see the top-level `AUTHORS.md` file.
+attribution, see the top-level ``AUTHORS.md`` file.
 '''
 
 
-AUTHOR_EMAIL = 'leycec@gmail.com'
+COPYRIGHT = '2014-2021 Beartype authors'
 '''
-Email address of the principal corresponding author (i.e., the principal author
-responding to public correspondence).
+Legally binding copyright line excluding the license-specific prefix (e.g.,
+``"Copyright (c)"``).
+
+For brevity, this string *only* lists authors explicitly assigned copyrights.
+For the list of all contributors regardless of copyright assignment or
+attribution, see the top-level ``AUTHORS.md`` file.
 '''
 
 # ....................{ METADATA ~ urls                   }....................
@@ -231,7 +243,7 @@ URL of this package's homepage.
 '''
 
 
-URL_DOWNLOAD = '{}/archive/{}.tar.gz'.format(URL_HOMEPAGE, VERSION)
+URL_DOWNLOAD = f'{URL_HOMEPAGE}/archive/{VERSION}.tar.gz'
 '''
 URL of the source tarball for the current version of this package.
 
@@ -243,17 +255,17 @@ have yet to be stabilized and hence tagged. Hence, this URL is typically valid
 '''
 
 
-URL_ISSUES = 'https://github.com/beartype/beartype/issues'
+URL_ISSUES = f'{URL_HOMEPAGE}/issues'
 '''
 URL of this package's issue tracker.
 '''
 
-# ....................{ METADATA ~ libs                   }....................
+# ....................{ METADATA ~ libs : runtime         }....................
 LIBS_RUNTIME_OPTIONAL = ()
 '''
-Optional runtime dependencies for this package defined as a tuple of
-:mod:`setuptools`-specific requirements strings of the format
-``{project_name} {comparison1}{version1},...,{comparisonN}{versionN}``, where:
+Optional runtime package dependencies as a tuple of :mod:`setuptools`-specific
+requirements strings of the format ``{project_name}
+{comparison1}{version1},...,{comparisonN}{versionN}``, where:
 
 * ``{project_name}`` is a :mod:`setuptools`-specific project name (e.g.,
   ``numpy``, ``scipy``).
@@ -261,28 +273,159 @@ Optional runtime dependencies for this package defined as a tuple of
   version comparison operators. As well as standard mathematical comparison
   operators (e.g., ``==``, ``>=``, ``<``), :mod:`setuptools` also supports the
   PEP 440-compliant "compatible release" operator ``~=`` more commonly denoted
-  by ``^`` in modern package managers (e.g., :mod:`poetry`, ``npm``); this
-  operator enables forward compatibility with all future versions of this
-  dependency known *not* to break backward compatibility, but should only be
-  applied to dependencies strictly following the semantic versioning contract.
+  by ``^`` in modern package managers (e.g., poetry, npm); this operator
+  enables forward compatibility with all future versions of this dependency
+  known *not* to break backward compatibility, but should only be applied to
+  dependencies strictly following the semantic versioning contract.
 * ``{version1}`` and ``{version1}`` are arbitrary version strings (e.g.,
   ``2020.2.16``, ``0.75a2``).
 '''
 
+# ....................{ METADATA ~ libs : test            }....................
+_LIBS_TESTTIME_MANDATORY_MYPY = (
+    # A *VERY* modern version of mypy is recommended. Even fairly recent older
+    # versions of mypy are significantly deficient with respect to error
+    # reporting to the point of uselessness.
+    'mypy >=0.800' ,
+) if not _IS_PYPY else ()
+'''
+**Mandatory mypy test-time package dependency** (i.e., dependencies required
+to test this package under :mod:`tox`) as a tuple of :mod:`setuptools`-specific
+requirements strings of the format ``{project_name}
+{comparison1}{version1},...,{comparisonN}{versionN}`` if the active Python
+interpreter is not PyPy *or* the empty tuple otherwise (i.e., if this
+interpreter is PyPy).
 
-LIBS_TESTTIME_MANDATORY = (
-    # pytest should ideally remain the only hard dependency for testing on
-    # local machines. While our testing regime optionally leverages third-party
-    # frameworks and pytest plugins (e.g., "tox", "pytest-xdist"), these
-    # dependencies are *NOT* required for simple testing.
-    #
-    # A relatively modern version of py.test is required.
+See Also
+----------
+:data:`LIBS_RUNTIME_OPTIONAL`
+    Further details.
+https://mypy.readthedocs.io/en/stable/faq.html#does-it-run-on-pypy
+    Official documentation discussing mypy's current incompatibility with PyPy
+    to presumably be fixed at some future point.
+'''
+
+
+LIBS_TESTTIME_MANDATORY_TOX = _LIBS_TESTTIME_MANDATORY_MYPY + (
+    # A fairly modern version of pytest is required.
     'pytest >=4.0.0',
 )
 '''
-Mandatory test-time dependencies for this package defined as a tuple of
-:mod:`setuptools`-specific requirements strings of the format
+**Mandatory tox test-time package dependencies** (i.e., dependencies required
+to test this package under :mod:`tox`) as a tuple of :mod:`setuptools`-specific
+requirements strings of the format ``{project_name}
+{comparison1}{version1},...,{comparisonN}{versionN}``.
+
+See Also
+----------
+:data:`LIBS_RUNTIME_OPTIONAL`
+    Further details.
+'''
+
+
+LIBS_TESTTIME_OPTIONAL = ()
+'''
+**Optional developer test-time package dependencies** (i.e., dependencies
+recommended to test this package with :mod:`tox` as a developer at the command
+line) as a tuple of :mod:`setuptools`-specific requirements strings of the
+format ``{project_name} {comparison1}{version1},...,{comparisonN}{versionN}``.
+
+See Also
+----------
+:data:`LIBS_RUNTIME_OPTIONAL`
+    Further details.
+'''
+
+
+LIBS_TESTTIME_MANDATORY = (
+    LIBS_TESTTIME_MANDATORY_TOX +
+    LIBS_TESTTIME_OPTIONAL + (
+        # A relatively modern version of tox is required.
+        'tox >=3.20.1',
+    )
+)
+'''
+**Mandatory developer test-time package dependencies** (i.e., dependencies
+required to test this package with :mod:`tox` as a developer at the command
+line) as a tuple of :mod:`setuptools`-specific requirements strings of the
+format ``{project_name} {comparison1}{version1},...,{comparisonN}{versionN}``.
+
+See Also
+----------
+:data:`LIBS_RUNTIME_OPTIONAL`
+    Further details.
+'''
+
+# ....................{ METADATA ~ libs : doc             }....................
+_LIB_VERSION_MIN_SPHINX = '3.4.3'
+'''
+Human-readable minimum version as a ``.``-delimited string of :mod:`sphinx`
+required to build package documentation.
+'''
+
+
+_LIB_VERSION_MIN_SPHINX_RTD_THEME = '0.5.1'
+'''
+Human-readable minimum version as a ``.``-delimited string of the **Read The
+Docs (RTD)-flavoured Sphinx theme** (i.e., :mod:`sphinx_rtd_theme`) optionally
+leveraged when building package documentation.
+'''
+
+
+LIBS_DOCTIME_MANDATORY = (
+    f'sphinx >={_LIB_VERSION_MIN_SPHINX}',
+)
+'''
+**Mandatory developer documentation build-time package dependencies** (i.e.,
+dependencies required to manually build documentation for this package as a
+developer at the command line) as a tuple of :mod:`setuptools`-specific
+requirements strings of the format ``{project_name}
+{comparison1}{version1},...,{comparisonN}{versionN}``.
+
+For flexibility, these dependencies are loosely relaxed to enable developers to
+build with *any* versions satisfying at least the bare minimum. For the same
+reason, optional documentation build-time package dependencies are omitted.
+Since our documentation build system emits a non-fatal warning for each missing
+optional dependency, omitting these optional dependencies here imposes no undue
+hardships while improving usability.
+
+See Also
+----------
+:data:`LIBS_RUNTIME_OPTIONAL`
+    Further details.
+'''
+
+
+LIBS_DOCTIME_MANDATORY_RTD = (
+    f'sphinx =={_LIB_VERSION_MIN_SPHINX}',
+    f'sphinx-rtd-theme =={_LIB_VERSION_MIN_SPHINX_RTD_THEME}',
+)
+'''
+**Mandatory Read The Docs (RTD) documentation build-time package dependencies**
+(i.e., dependencies required to automatically build documentation for this
+package from the third-party RTD hosting service) as a tuple of
+:mod:`setuptools`-specific requirements strings of the format ``{project_name}
+{comparison1}{version1},...,{comparisonN}{versionN}``.
+
+For consistency, these dependencies are strictly constrained to force RTD to
+build against a single well-tested configuration known to work reliably.
+
+See Also
+----------
+:data:`LIBS_RUNTIME_OPTIONAL`
+    Further details.
+'''
+
+# ....................{ METADATA ~ libs : dev             }....................
+LIBS_DEVELOPER_MANDATORY = LIBS_TESTTIME_MANDATORY + LIBS_DOCTIME_MANDATORY
+'''
+**Mandatory developer package dependencies** (i.e., dependencies required to
+develop and meaningfully contribute pull requests for this package) as a tuple
+of :mod:`setuptools`-specific requirements strings of the format
 ``{project_name} {comparison1}{version1},...,{comparisonN}{versionN}``.
+
+This tuple includes all mandatory test- and documentation build-time package
+dependencies and is thus a convenient shorthand for those lower-level tuples.
 
 See Also
 ----------
